/*****************************************************************************/
/* File: Camera.h                                                            */
/*                                                                           */
/* Description:                                                              */
/*                                                                           */
/* The camera object is responsible for transforming the world through the   */
/* view matrix and perspective projection matrix. Controlling the camera     */
/* through the supplied operations will alter the view of the world.         */
/*                                                                           */
/* All portions Copyright (c) Austin Krauss, 2003.                           */
/*****************************************************************************/

                           /* MODIFICATION LOG */
/*****************************************************************************/
/*  Date    *  Rev # *  Who     *   Comment                                  */
/*---------------------------------------------------------------------------*/
/* 5/10/03  *   00   *  Austin  * Created.                                   */
/*****************************************************************************/

/*****************************************************************************/
/*                                 HEADERS                                   */  
/*****************************************************************************/

#if !defined(CAMERA_H)
#define CAMERA_H

#ifdef _DEBUG
#include "Logger.h"
#endif 

/*****************************************************************************/
/*                            CLASS INTERFACE                                */
/*****************************************************************************/

namespace MidwayMath {

  class Camera : public CD3DCamera {

    public:

      // Constructors / Destructors
      Camera(D3DDISPLAYMODE *d, D3DXVECTOR3 pos, D3DXVECTOR3 lookAt) 
        { mpDisplayMode = d, Initialize(pos, lookAt); }
    #ifdef _DEBUG
      Camera(Logger *l, D3DDISPLAYMODE *d, D3DXVECTOR3 pos, D3DXVECTOR3 lookAt) 
        { mpDisplayMode = d; mpLog = l; Initialize(pos, lookAt); }
    #endif
     ~Camera() { }

      // Operations
      inline void SetEyePt(D3DXVECTOR3 v) { SetViewParams(v, GetLookatPt(), GetUpVec()); }
      inline void SetLookatPt(D3DXVECTOR3 v) { SetViewParams(GetEyePt(), v, GetUpVec()); }
    
    private:
    
    #ifdef _DEBUG
      Logger *mpLog;
    #endif
  
      D3DDISPLAYMODE *mpDisplayMode;

      // Helper functions
      inline void Initialize(D3DXVECTOR3 position, D3DXVECTOR3 lookAt);
  };

  /****************************************************************************/
  /*  FUNCTION:                                                               */
  /*                                                                          */
  /*      Initialize                                                          */
  /*                                                                          */
  /*  PARAMETERS:                                                             */
  /*                                                                          */
  /*      position                                                            */
  /*          The position vector of the camera.                              */
  /*                                                                          */
  /*      lookAt                                                              */
  /*          The look at point of the camera.                                */
  /*                                                                          */
  /*  DESCRIPTION:                                                            */
  /*                                                                          */
  /*      Called by constructor to initialize the project perspective matrix  */
  /*      and the view matrix.                                                */
  /*                                                                          */
  /*  RETURNS:                                                                */
  /*                                                                          */
  /****************************************************************************/
  
  inline void Camera::Initialize(D3DXVECTOR3 position, D3DXVECTOR3 lookAt) {
  
    //setup a perspective projection matrix
    float fov_y        = (float)D3DXToRadian(60);
    float Aspect_Ratio = (float)mpDisplayMode->Width / (float)mpDisplayMode->Height;
    float z_near       = 1.0f;
    float z_far        = 1500.0f;
    SetProjParams(fov_y, Aspect_Ratio, z_near, z_far);
  
    //setup the camera
    SetViewParams(position, lookAt, D3DXVECTOR3(0,1,0));
  }

} // End Namespace
#endif
