/*****************************************************************************/
/* File: Logger.h                                                            */
/*                                                                           */
/* Description:                                                              */
/*                                                                           */
/* The Logger object will serve as a file I/O device used to log diagnostic  */
/* information generated by applications.                                    */
/*                                                                           */
/* All portions Copyright (c) Austin Krauss, 2002-2003.                      */
/*****************************************************************************/

                           /* MODIFICATION LOG */
/*****************************************************************************/
/*  Date    *  Rev # *  Who     *   Comment                                  */
/*---------------------------------------------------------------------------*/
/* 4/15/03  *   03   *  Austin  * Added LOGnd macro for non-dynamic objects. */
/*---------------------------------------------------------------------------*/
/* 4/12/03  *   02   *  Austin  * Made Log() inline. Made Load() into a      */
/*          *        *          * constructor.                               */
/*---------------------------------------------------------------------------*/
/* 4/01/03  *   01   *  Austin  * Updated for the Speedy Delivery Project.   */
/*---------------------------------------------------------------------------*/
/* 12/31/02 *   00   *  Austin  * Created.                                   */
/*****************************************************************************/

/*****************************************************************************/
/*                                 HEADERS                                   */  
/*****************************************************************************/

#if !defined(LOGGER_H)
#define LOGGER_H

#include <fstream>
#include <ctime>

/*****************************************************************************/
/*                                CONSTANTS                                  */  
/*****************************************************************************/

#define LOG(l) l->Log(__FILE__, __LINE__) // Silently pass FILE and LINE
#define LOGnd(l) l.Log(__FILE__, __LINE__) // Silently pass FILE and LINE

/*****************************************************************************/
/*                            CLASS INTERFACE                                */
/*****************************************************************************/

namespace MidwayMath {

  class Logger {

    public:

      Logger() { }    
      Logger(const char *cpFileName);    
      ~Logger();

      inline std::ofstream& Log(char *cpFileName, int iLine);
  
    private:

      std::ofstream mOutStream;

      inline const char* TruncateFileName(const char *const cpFileName);
      inline char* GetTimeAsString();
  };

  /*****************************************************************************/
  /*  FUNCTION:                                                                */
  /*                                                                           */
  /*      GetTimeAsString                                                      */
  /*                                                                           */
  /*  PARAMETERS:                                                              */
  /*                                                                           */
  /*  DESCRIPTION:                                                             */
  /*                                                                           */
  /*      Get the time as a character pointer. See the ctime() documentation   */
  /*      the actual format returned.                                          */
  /*                                                                           */
  /*  RETURNS:                                                                 */
  /*                                                                           */
  /*      A character pointer pointing to the string.                          */
  /*                                                                           */
  /*****************************************************************************/

  inline char* Logger::GetTimeAsString() {

  time_t secs;
  char *t;
    
    time(&secs);
    t = ctime(&secs);
    t[24] = '\0'; //strip the newline

    return t;
  }

  /*****************************************************************************/
  /*  FUNCTION:                                                                */
  /*                                                                           */
  /*      TruncateFileName                                                     */
  /*                                                                           */
  /*  PARAMETERS:                                                              */
  /*                                                                           */
  /*      cpFileName                                                           */
  /*          A const pointer to a const string representation of the filename.*/
  /*                                                                           */
  /*  DESCRIPTION:                                                             */
  /*                                                                           */
  /*      Makes sure that the buffer isn't overrun.                            */
  /*                                                                           */
  /*  RETURNS:                                                                 */
  /*                                                                           */
  /*      const char*                                                          */
  /*          A const character pointer pointing to a filename string without  */
  /*          the prefixed path.                                               */
  /*                                                                           */
  /*****************************************************************************/

  inline const char* Logger::TruncateFileName(const char *const cpFileName) {

  const char *p = cpFileName;
      
    p += strlen(cpFileName) - 1;
    
    while(*p != '\\' && p != cpFileName)
      p--;
    
    return(++p);
  }

  /*****************************************************************************/
  /*  FUNCTION:                                                                */
  /*                                                                           */
  /*      Log                                                                  */
  /*                                                                           */
  /*  PARAMETERS:                                                              */
  /*                                                                           */
  /*      cpFileName                                                           */
  /*          A pointer to a string representation of the filename.            */
  /*                                                                           */
  /*      iLine                                                                */
  /*          The line number of the file to be logged.                        */
  /*                                                                           */
  /*  DESCRIPTION:                                                             */
  /*                                                                           */
  /*      Logs a header message with the filename and line number.             */
  /*                                                                           */
  /*  RETURNS:                                                                 */
  /*                                                                           */
  /*      std::ofstream&                                                       */
  /*          A stream pointer into the log file for the caller to log         */
  /*          additional messages.                                             */
  /*                                                                           */
  /*****************************************************************************/

  inline std::ofstream& Logger::Log(char *cpFileName, int iLine) {

    mOutStream << "<-" << GetTimeAsString() << " " << TruncateFileName(cpFileName) << "(" << iLine << ")->\n  ";
    return mOutStream;
  }            

} // End Namespace
#endif
