/*****************************************************************************/
/* File: MidwayMathApp.h                                                     */
/*                                                                           */
/* Description:                                                              */
/*                                                                           */
/* Application object for the Midway Math project. Creates subsystem         */
/* objects and intializes them as appropriate. Contains Windows Message      */
/* Handler to pass any recieved keystrokes to related objects.               */
/* Creates a global application class which is passed any Windows' Messages  */
/* recieved from the message callback function.                              */
/*                                                                           */
/* All portions Copyright (c) Austin Krauss, 2003.                           */
/*****************************************************************************/

                           /* MODIFICATION LOG */
/*****************************************************************************/
/*  Date    *  Rev # *  Who     *   Comment                                  */
/*---------------------------------------------------------------------------*/
/* 11/18/03 *   00   *  Austin  * Created.                                   */
/*****************************************************************************/

/*****************************************************************************/
/*                                 HEADERS                                   */  
/*****************************************************************************/

#if !defined(MIDWAYMATHAPP_H)
#define MIDWAYMATHAPP_H

#include <windows.h>
#include "Dxerr8.h"
#include "dsutil.h"

#include "Renderer.h"
#include "ModelFileLoader.h"
#include "Camera.h"
#include "Ball.h"
#include "Crate.h"
#include "InputManager.h"
#include "MidwayMathUI.h"

#ifdef _DEBUG
#include "Logger.h"
#endif 


/*****************************************************************************/
/*                               EXTERNS                                     */
/*****************************************************************************/

extern LRESULT CALLBACK WinMessageHandler(HWND, UINT, WPARAM, LPARAM);

/*****************************************************************************/
/*                            CLASS INTERFACE                                */
/*****************************************************************************/

namespace MidwayMath {

  class MidwayMathApp {

    public:

      // Constants
      static const long  MIDWAY_PHYSICS_COUNT;
      static const float MIDWAY_PHYSICS_GRAVITY;
      
      static const long MIDWAY_CRATE_COUNT;
      static const char* const MIDWAY_MODEL_CRATE1;
      static const char* const MIDWAY_MODEL_CRATE3;
      static const char* const MIDWAY_MODEL_CRATE4;
      static const char* const MIDWAY_MODEL_CRATE5;
      static const char* const MIDWAY_MODEL_GROUND;
      static const char* const MIDWAY_MODEL_BALL;

      enum {MIDWAY_SOUND_COUNT = 8};
      static const char* const MIDWAY_SOUND_GAMEEND;
      static const char* const MIDWAY_SOUND_RIGHTANSWER;
      static const char* const MIDWAY_SOUND_WRONGANSWER;
      static const char* const MIDWAY_SOUND_BIRD;
      static const char* const MIDWAY_SOUND_HIT1;
      static const char* const MIDWAY_SOUND_HIT2;
      static const char* const MIDWAY_SOUND_HIT3;
      static const char* const MIDWAY_SOUND_THUD;

      static const long MIDWAY_SOUND_INDEX_GAMEEND;
      static const long MIDWAY_SOUND_INDEX_RIGHTANSWER;
      static const long MIDWAY_SOUND_INDEX_WRONGANSWER;
      static const long MIDWAY_SOUND_INDEX_BIRD;
      static const long MIDWAY_SOUND_INDEX_HIT1;
      static const long MIDWAY_SOUND_INDEX_HIT2;
      static const long MIDWAY_SOUND_INDEX_HIT3;
      static const long MIDWAY_SOUND_INDEX_THUD;
      
      // Math specific
      static const char MIDWAY_MATH_PLUS_OP;
      static const char MIDWAY_MATH_MINUS_OP;

      // Constructors / Destructors
      MidwayMathApp() { }
      MidwayMathApp(HINSTANCE);
    #ifdef _DEBUG
      MidwayMathApp(Logger *, HINSTANCE);
    #endif
      ~MidwayMathApp();
    
      // Operations
      void    Run();
      LRESULT MessageHandler(HWND, UINT, WPARAM, LPARAM);
      void HandleRightAnswer(Crate*);
      void HandleWrongAnswer(Crate*);
      void HandleCollapse(Crate*);
      void HandleCratesCollide();
    
    private:

    #ifdef _DEBUG
      Logger *mpLog;
    #endif
     
      // States
      static const long MIDWAY_STATE_GAME;
      static const long MIDWAY_STATE_GAME_OVER;
      static const long MIDWAY_STATE_MENU;
      static const long MIDWAY_STATE_PAUSE;

      // Windows handles
      HWND      mhWnd;
      HINSTANCE mhInstance;
      
      // Subsystem objects
      Renderer       *mpRenderer;
      SceneGraphNode *mpRootNode;
      InputManager   *mpInputManager;
      MidwayMathUI   *mpUI;   
      Camera         *mpCamera;
      neSimulator    *mpPhysics;
      CSoundManager  *mpSoundManager;
      
      // Current menu
      long mlCurrentMenu;

      // Current state
      long mlCurrentState;

      // Game-specific objects
      Ball *mpBall;
      CSound *testSound;
      CSound *mpaSounds[MIDWAY_SOUND_COUNT];
      
      // Game-specific variables
      bool mbRightAnswer;
      int miRightAnswers;
      int miWrongAnswers;
      int miNumThrows;
      bool mbEndSound;
      
      // Helper functions
      void Initialize();
      void InitializeMenus();
      void InitializePhysics();
      void InitializeGameObjects();
      void InitializeSounds();

      void ResetGameObjects();
      void UpdatePhysics();
      void ReadPlayerInput();
      void HandleMenuSelect();
      void HandleMouseMessage(int, float); 
      void HandleMenuPause();
      void HandleGameOver();
      bool CreateMathProblem();
      inline void SetRightAnswer(bool a) { mbRightAnswer = a; }
      inline bool GetRightAnswer() { return mbRightAnswer; }
  };

} // End Namespace
#endif
