/*****************************************************************************/
/* File: Renderer.h                                                          */
/*                                                                           */
/* Description:                                                              */
/*                                                                           */
/* Object responsible for the main rendering subsystem. Creates Direct3D     */
/* rendering objects from given parameters of resolution and fullscreen mode.*/
/* Contains access and utility functions for objects to use for rendering    */
/* their geometry. Contains a reference to the root Scene Graph node for     */
/* rendering geometry.                                                       */
/*                                                                           */
/* All portions Copyright (c) Austin Krauss, 2003.                           */
/*****************************************************************************/

                           /* MODIFICATION LOG */
/*****************************************************************************/
/*  Date    *  Rev # *  Who     *   Comment                                  */
/*---------------------------------------------------------------------------*/
/* 6/15/03  *   01   *  Austin  * Misc. cleanup                              */
/*---------------------------------------------------------------------------*/
/* 4/20/03  *   00   *  Austin  * Created.                                   */
/*****************************************************************************/

/*****************************************************************************/
/*                                 HEADERS                                   */  
/*****************************************************************************/

#if !defined(RENDERER_H)
#define RENDERER_H

#include <windows.h>
#include <d3d8.h>
#include <vector>
#include "d3dfont.h"
#include "dxutil.h"
#include "d3dutil.h"
#include "d3dx8math.h"

#include "SceneGraphNode.h"
#include "Camera.h"

#ifdef _DEBUG
#include "Logger.h"
#endif

/*****************************************************************************/
/*                            CLASS INTERFACE                                */
/*****************************************************************************/

namespace MidwayMath {

  class Renderer {

    public:

      //Constants
      static const int RENDERER_RES_640X480;
      static const int RENDERER_RES_800X600;
      static const int RENDERER_RES_1024X768;
      static const int RENDERER_RES_1280X1024;

      //Constructors / Destructors
      Renderer() { }
      
    #ifdef _DEBUG  
       Renderer(Logger *pLog, HWND hWnd, int res = RENDERER_RES_640X480, bool fs = true);
    #endif
       Renderer(HWND hWnd, int res = RENDERER_RES_640X480, bool fs = true); 
      ~Renderer();
    
      //Operations
      inline HWND GetHWND()                 { return mhWnd; }
      inline LPDIRECT3DDEVICE8  GetDevice() { return mpD3D8Device; }
      void GetResolution(int *width, int *height);

      inline SceneGraphNode* GetRootNode()  { return mpRootNode; }
      inline void SetRootNode(SceneGraphNode *n) { mpRootNode = n; }
      
      inline Camera* GetCamera()       { return mpCamera; }
      inline void SetCamera(Camera *c) { mpCamera = c; }

      inline void SetDebug(bool d) { mbShowStatistics = d; }
      inline bool GetDebug()       { return mbShowStatistics; }

      void Render();
      void BeginRenderMenu();
      void EndRenderMenu();
    
    private:

    #ifdef _DEBUG
      Logger *mpLog;
    #endif
     
      //Internal States
      bool   mbShowStatistics;
      bool   mbFullScreen;
      int    miResolution;
  
      //Camera
      Camera *mpCamera; 

      //Scene Graph Root node
      SceneGraphNode *mpRootNode;
      
      //Windows & DirectX Specific
      HWND              mhWnd;
      LPDIRECT3D8       mpD3D8Object;
      LPDIRECT3DDEVICE8 mpD3D8Device;
      std::vector <D3DDISPLAYMODE> mDisplayModeList; // List of all available modes we're interested in
  
      //Statistics
      CD3DFont* mpStatsFont;   
      char      mcStats[2048];
      char      mcModeInfo[64];
      char      mcDeviceInfo[1280];

      //Helper functions
      void Initialize(int);
      void BuildDisplayModeList(UINT);
      void CalculateStatistics();
      void DrawAxis();

      bool DrawSkybox(D3DXMATRIX *view, float Size);
      LPDIRECT3DTEXTURE8 mpSkybox[6];
  };

} // End Namespace
#endif
