/*****************************************************************************/
/* File: SceneGraphNode.h                                                    */
/*                                                                           */
/* Description:                                                              */
/*                                                                           */
/* Abstract base class used for representing a node in a Scene Graph.        */
/*                                                                           */
/* All portions Copyright (c) Austin Krauss, 2003.                           */
/*****************************************************************************/

                           /* MODIFICATION LOG */
/*****************************************************************************/
/*  Date    *  Rev # *  Who     *   Comment                                  */
/*---------------------------------------------------------------------------*/
/* 6/15/03  *   02   *  Austin  * Misc. cleanup                              */
/*---------------------------------------------------------------------------*/
/* 5/10/03  *   01   *  Austin  * Renamed to SceneGraphNode, added operations*/
/*          *        *          * from 2nd iteration design.                 */
/*---------------------------------------------------------------------------*/
/* 4/27/03  *   00   *  Austin  * Created.                                   */
/*****************************************************************************/

/*****************************************************************************/
/*                                 HEADERS                                   */  
/*****************************************************************************/

#if !defined(SCENEGRAPHNODE_H)
#define SCENEGRAPHNODE_H

#include <vector>
#include <string>

#ifdef _DEBUG
#include "Logger.h"
#endif 

/*****************************************************************************/
/*                            CLASS INTERFACE                                */
/*****************************************************************************/

namespace MidwayMath {

  class SceneGraphNode {

    public:

      //Pseudo-Constructors 
      inline void Initialize()        
        { mName = "NULL"; mpParent = NULL; }
      
      inline void Initialize(std::string n) 
        { mName = n; mpParent = NULL; }
      
      inline void Initialize(std::string n, SceneGraphNode *p) 
        { mName = n; mpParent = p; }
    
    #ifdef _DEBUG  
      inline void Initialize(std::string n, Logger *l) 
        { mName = n; mpLog = l; mpParent = NULL; }
      
      inline void Initialize(std::string n, Logger *l, SceneGraphNode *p) 
        { mName = n; mpLog = l; mpParent = p; }
    #endif

      //Operations
      inline int    GetNumberOfChildren()         { return mChildren.size(); }
      inline int    GetNumberOfSubtreeNodes();
      inline std::string GetName()                { return mName; }
      
      inline void  AttachChild(SceneGraphNode *n) { mChildren.push_back(n); }
      
      inline void  RemoveChild(int index)         { mChildren.erase(mChildren.begin() + index); }
      inline void  RemoveChild(std::string);
      inline void  RemoveChild(SceneGraphNode *node) { RemoveChild(node->GetName()); }
      
      inline SceneGraphNode* GetParent() { return mpParent; }
      inline SceneGraphNode* GetChild(std::string);
      inline SceneGraphNode* GetChild(int index) { return mChildren[index]; }
    
      inline friend bool operator ==(const SceneGraphNode&, const SceneGraphNode&);  
    
      virtual void Update() = 0;
      virtual void Draw()   = 0;

    protected:

    #ifdef _DEBUG
      Logger *mpLog;
    #endif
  
    private:

      std::string mName;
      SceneGraphNode *mpParent;
      std::vector<SceneGraphNode*> mChildren;
  };

  /****************************************************************************/
  /*  FUNCTION:                                                               */
  /*                                                                          */
  /*      RemoveChild                                                         */
  /*                                                                          */
  /*  PARAMETERS:                                                             */
  /*                                                                          */
  /*      name                                                                */
  /*          The name of the Node that shall be removed from the children    */
  /*          list.                                                           */
  /*                                                                          */
  /*  DESCRIPTION:                                                            */
  /*                                                                          */
  /*      Removes the child having name from the children list. Does nothing  */
  /*      if a child does not exist with the given name.                      */
  /*                                                                          */
  /*  RETURNS:                                                                */
  /*                                                                          */
  /****************************************************************************/

  inline void SceneGraphNode::RemoveChild(std::string name) {

  std::vector <SceneGraphNode*>::iterator child;
  
    for(child = mChildren.begin(); child != mChildren.end(); child++) {

      if(mName == name) { 
        
        mChildren.erase(child);
        return;
      }
    }
  }

  /****************************************************************************/
  /*  FUNCTION:                                                               */
  /*                                                                          */
  /*      GetChild                                                            */
  /*                                                                          */
  /*  PARAMETERS:                                                             */
  /*                                                                          */
  /*      name                                                                */
  /*          The name of the Node that will be returned.                     */
  /*                                                                          */
  /*  DESCRIPTION:                                                            */
  /*                                                                          */
  /*      Returns a pointer to the node having the name specified.            */
  /*                                                                          */
  /*  RETURNS:                                                                */
  /*                                                                          */
  /*      If found, returns a pointer to a SceneGraphNode, NULL otherwise.    */
  /*                                                                          */
  /****************************************************************************/

  inline SceneGraphNode* SceneGraphNode::GetChild(std::string name) {

  std::vector <SceneGraphNode*>::iterator child;
  
    for(child = mChildren.begin(); child != mChildren.end(); child++) {

      if(mName == name) { 
        
        return *child;
      }
    }
  return NULL;
  }
  
  /****************************************************************************/
  /*  FUNCTION:                                                               */
  /*                                                                          */
  /*      GetNumberOfSubtreeNodes                                             */
  /*                                                                          */
  /*  PARAMETERS:                                                             */
  /*                                                                          */
  /*  DESCRIPTION:                                                            */
  /*                                                                          */
  /*      Counts the number of nodes in this subtree. By recursively calling  */
  /*      this function for each child.                                       */
  /*                                                                          */
  /*  RETURNS:                                                                */
  /*                                                                          */
  /*      The number of nodes in this subtree.                                */
  /*                                                                          */
  /****************************************************************************/
  
  inline int SceneGraphNode::GetNumberOfSubtreeNodes() {

  int num = GetNumberOfChildren();
  std::vector <SceneGraphNode*>::iterator child;
  
    for(child = mChildren.begin(); child != mChildren.end(); child++) 
      num += (*child)->GetNumberOfSubtreeNodes();
  
  return num;
  }


  /****************************************************************************/
  /*  FUNCTION:                                                               */
  /*                                                                          */
  /*      operator ==                                                         */
  /*                                                                          */
  /*  PARAMETERS:                                                             */
  /*                                                                          */
  /*  DESCRIPTION:                                                            */
  /*                                                                          */
  /*      Tests to see if two nodes are equivalent by comparing their names   */
  /*      and parents.                                                        */
  /*                                                                          */
  /*  RETURNS:                                                                */
  /*                                                                          */
  /*      True if the two nodes are equivalent, false otherwise.              */
  /*                                                                          */
  /****************************************************************************/
  
  inline bool operator ==(const SceneGraphNode& n1, const SceneGraphNode& n2) {

    if(n1.mName == n2.mName)
      if(n1.mpParent == n2.mpParent)
        return true;    

    return false;
  }  

} // End Namespace
#endif
