/*****************************************************************************/
/* File: Ball.cpp                                                            */
/*                                                                           */
/* Description:                                                              */
/*                                                                           */
/* All portions Copyright (c) Austin Krauss, 2003.                           */
/*****************************************************************************/

                           /* MODIFICATION LOG */
/*****************************************************************************/
/*  Date    *  Rev # *  Who     *   Comment                                  */
/*---------------------------------------------------------------------------*/
/* 12/14/03 *   00   *  Austin  * Created.                                   */
/*****************************************************************************/

/*****************************************************************************/
/*                                 HEADERS                                   */  
/*****************************************************************************/

#include "Ball.h"
#include "ModelFileLoader.h"

using namespace MidwayMath;

/*****************************************************************************/
/*                                CONSTANTS                                  */  
/*****************************************************************************/

const float Ball::BALL_PHYSICS_MASS     = 1.0f;
const float Ball::BALL_PHYSICS_VELOCITY = 500.0f;

/*****************************************************************************/
/*                               CONSTRUCTORS                                */  
/*****************************************************************************/

Ball::Ball(Logger *l, Renderer *r, neSimulator *sim, SceneGraphNode *parent, const char *file) 
  :ModelNode(l, r, "ball", parent),
   mpPhysics(sim) {

LPD3DXMESH mesh;

// bounding sphere stuff
BYTE* pVertices;
DWORD fvf;
DWORD numVert;
D3DXVECTOR3 center;
float radius, diameter;
neGeometry *geom;

  // Load the mesh
  ModelFileLoader *loader = new ModelFileLoader(l, r, file, this);
  loader->Load();
  delete loader;

  // Compute the sphere dimensions
  mesh = GetMesh();
  fvf = mesh->GetFVF();
  numVert = mesh->GetNumVertices();

  mesh->LockVertexBuffer(D3DLOCK_READONLY, &pVertices);

    D3DXComputeBoundingSphere(pVertices,
                              numVert,
                              fvf,
                              &center,
                              &radius);

  mesh->UnlockVertexBuffer();

  diameter = radius * 2;
    
  // Create the rigid body
  mpRigidBody = mpPhysics->CreateRigidBody();
  
  // Create the bounding sphere
  geom = mpRigidBody->AddGeometry();
  geom->SetSphereDiameter(diameter);
  mpRigidBody->UpdateBoundingInfo();
  
  // Set the mass
  mpRigidBody->SetInertiaTensor(neSphereInertiaTensor(diameter, BALL_PHYSICS_MASS));
  mpRigidBody->SetMass(BALL_PHYSICS_MASS);
}

/*****************************************************************************/
/*                               DESTRUCTOR                                  */  
/*****************************************************************************/

Ball::~Ball() {

  // can't free the model mesh since we may not own it
  // Free the physics stuff  
  mpPhysics->FreeRigidBody(mpRigidBody);
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      Throw                                                                */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void Ball::Throw(D3DXVECTOR3 dir) {

neV3 direction;

  direction.Set(dir.x, dir.y, dir.z);
  mpRigidBody->SetVelocity(direction * BALL_PHYSICS_VELOCITY);
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      Update                                                               */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void Ball::Update() {

neV3 pos;
neM3 rot;

  pos = mpRigidBody->GetPos();
  rot = mpRigidBody->GetRotationM3();

  mLocalRotation._11 = rot[0][0]; mLocalRotation._12 = rot[0][1]; mLocalRotation._13 = rot[0][2];
  mLocalRotation._21 = rot[1][0]; mLocalRotation._22 = rot[1][1]; mLocalRotation._23 = rot[1][2];
  mLocalRotation._31 = rot[2][0]; mLocalRotation._32 = rot[2][1]; mLocalRotation._33 = rot[2][2];

  mLocalTranslation.x = pos[0];
  mLocalTranslation.y = pos[1];
  mLocalTranslation.z = pos[2];

  ModelNode::Update();
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      GetPosition                                                          */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

D3DXVECTOR3 Ball::GetPosition() {

neV3 pos;

  pos = mpRigidBody->GetPos();

  return(D3DXVECTOR3(pos[0], pos[1], pos[2]));
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      SetPosition                                                          */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void Ball::SetPosition(D3DXVECTOR3 p) {

neV3 pos;
neV3 vel;

  vel.SetZero();
  mpRigidBody->SetVelocity(vel);
  mpRigidBody->SetAngularMomentum(vel);
  mpRigidBody->SetForce(vel);
  mpRigidBody->SetTorque(vel);

  pos.Set(p.x, p.y, p.z);
  mpRigidBody->SetPos(pos);
}
