/*****************************************************************************/
/* File: Crate.cpp                                                           */
/*                                                                           */
/* Description:                                                              */
/*                                                                           */
/* All portions Copyright (c) Austin Krauss, 2003.                           */
/*****************************************************************************/

                           /* MODIFICATION LOG */
/*****************************************************************************/
/*  Date    *  Rev # *  Who     *   Comment                                  */
/*---------------------------------------------------------------------------*/
/* 12/12/03 *   00   *  Austin  * Created.                                   */
/*****************************************************************************/

/*****************************************************************************/
/*                                 HEADERS                                   */  
/*****************************************************************************/

#include "Crate.h"
#include "ModelFileLoader.h"

using namespace MidwayMath;


/*****************************************************************************/
/*                                CONSTANTS                                  */  
/*****************************************************************************/

const float Crate::CRATE_PHYSICS_MASS = 15.0f;

/*****************************************************************************/
/*                               CONSTRUCTORS                                */  
/*****************************************************************************/

Crate::Crate(Logger *l, Renderer *r, neSimulator *sim, SceneGraphNode *parent, const char *file)
  :ModelNode(l, r, "crate", parent),
   mpPhysics(sim),
   miNumber(-1) {
       
  // Load the mesh for this node
  ModelFileLoader *loader = new ModelFileLoader(l, r, file, this);
  loader->Load();
  delete loader;

  Initialize();
}

Crate::Crate(Logger *l, Renderer *r, neSimulator *sim, SceneGraphNode *parent, LPD3DXMESH mesh) 
  :ModelNode(l, r, "crate", parent),
   mpPhysics(sim),
   miNumber(-1) {

  // We were passed a mesh, set it as ours 
  SetMesh(mesh);
  Initialize();
}

/*****************************************************************************/
/*                               DESTRUCTOR                                  */  
/*****************************************************************************/

Crate::~Crate() {

  // can't free the model mesh since we may not own it
  // Free the physics stuff  
  mpPhysics->FreeRigidBody(mpRigidBody);
  delete mpFont;
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      Initialize                                                           */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void Crate::Initialize() {

LPD3DXMESH mesh;
DWORD fvf, numVert;
BYTE* pVertices;

// Bounding box stuff
D3DXVECTOR3 min, max;
neGeometry *geom;
float x, y, z;

  // Time to set up the physics for our crate
  mesh = GetMesh();
  fvf = mesh->GetFVF();
  numVert = mesh->GetNumVertices();

  mesh->LockVertexBuffer(D3DLOCK_READONLY, &pVertices);

    // Forgo the error checking due to time constaints :-)
    D3DXComputeBoundingBox(pVertices,
                           numVert,
                           fvf,
                           &min,
                           &max);
  
  mesh->UnlockVertexBuffer();

  x = max.x - min.x; // Width
  y = max.y - min.y; // Length
  z = max.z - min.z; // Depth

  // Create a rigid body
  mpRigidBody = mpPhysics->CreateRigidBody();
  mpRigidBody->SetUserData((unsigned int) this);
  
  // Define the bounding box
  geom = mpRigidBody->AddGeometry();
  geom->SetBoxSize(x, y, z);
  mpRigidBody->UpdateBoundingInfo();

  // Set the mass of this crate
  mpRigidBody->SetInertiaTensor(neBoxInertiaTensor(x, y, z, CRATE_PHYSICS_MASS));
  mpRigidBody->SetMass(CRATE_PHYSICS_MASS);

  // Number font
  mpFont = new CD3DFont( _T("Impact"), 48, 0L ); 
  mpFont->InitDeviceObjects(mpRenderer->GetDevice());
  mpFont->RestoreDeviceObjects();
  D3DUtil_InitMaterial(mGameFontMaterial, 1.0f, 0.0f, 0.0f, 0.8f);
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      Update                                                               */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void Crate::Update() {

neV3 pos;
neM3 rot;
  
  pos = mpRigidBody->GetPos();
  rot = mpRigidBody->GetRotationM3();

  // Convert Tokamak's position to our internal forms
  mLocalRotation._11 = rot[0][0]; mLocalRotation._12 = rot[0][1]; mLocalRotation._13 = rot[0][2];
  mLocalRotation._21 = rot[1][0]; mLocalRotation._22 = rot[1][1]; mLocalRotation._23 = rot[1][2];
  mLocalRotation._31 = rot[2][0]; mLocalRotation._32 = rot[2][1]; mLocalRotation._33 = rot[2][2];

  mLocalTranslation.x = pos[0];
  mLocalTranslation.y = pos[1];
  mLocalTranslation.z = pos[2];

  ModelNode::Update();
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      Draw                                                                 */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void Crate::Draw() {

D3DXVECTOR3 position;
D3DXMATRIX matWorld;
char num[8];

  ModelNode::Draw();

  if(miNumber < 0) return;

  sprintf(num, "%d", miNumber);
  
  position = GetPosition();

  position.x -= 2.0f;
  position.y += 2.0f;
  position.z -= 10.0f;

  D3DXMatrixScaling(&matWorld, 6.0f, 6.0f, 6.0f);
  matWorld._41 = position.x;
  matWorld._42 = position.y;
  matWorld._43 = position.z;
  mpRenderer->GetDevice()->SetTransform(D3DTS_WORLD, &matWorld);

  mpRenderer->GetDevice()->SetMaterial(&mGameFontMaterial);
  mpFont->Render3DText(num, D3DFONT_CENTERED|D3DFONT_TWOSIDED);
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      GetPosition                                                          */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

D3DXVECTOR3 Crate::GetPosition() {

neV3 pos;

  pos = mpRigidBody->GetPos();

  return(D3DXVECTOR3(pos[0], pos[1], pos[2]));
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      SetPosition                                                          */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void Crate::SetPosition(D3DXVECTOR3 p) {

neV3 pos;
neV3 vel;
neM3 rot;

  vel.SetZero();
  rot.SetIdentity();
  mpRigidBody->SetVelocity(vel);
  mpRigidBody->SetAngularMomentum(vel);
  mpRigidBody->SetForce(vel);
  mpRigidBody->SetTorque(vel);
  mpRigidBody->SetRotation(rot);
  
  pos.Set(p.x, p.y, p.z);
  mpRigidBody->SetPos(pos);
}
