/*****************************************************************************/
/* File: InputManager.cpp                                                    */
/*                                                                           */
/* Description:                                                              */
/*                                                                           */
/* Processes user input for both menus and game play. Created by the main    */
/* application object (MidwayMathApp); this object is polled for input   */
/* returning one of the predefined constants.                                */
/*                                                                           */
/* All portions Copyright (c) Austin Krauss, 2003.                           */
/*****************************************************************************/

                           /* MODIFICATION LOG */
/*****************************************************************************/
/*  Date    *  Rev # *  Who     *   Comment                                  */
/*---------------------------------------------------------------------------*/
/* 7/17/03  *   00   *  Austin  * Created.                                   */
/*****************************************************************************/

/*****************************************************************************/
/*                                 HEADERS                                   */  
/*****************************************************************************/

#include "InputManager.h"

using namespace MidwayMath;

/*****************************************************************************/
/*                                CONSTANTS                                  */  
/*****************************************************************************/

// {5D4E34A2-757B-4755-AFEA-4228802E523A}
static const GUID sAppGuid = 
  { 0x5d4e34a2, 0x757b, 0x4755, { 0xaf, 0xea, 0x42, 0x28, 0x80, 0x2e, 0x52, 0x3a } };

  // Actions used by this app
DIACTION gGameAction[] =
  {
    //Genre-defined virtual buttons 
    {InputManager::INPUT_PLAYER_LOOKUD, DIMOUSE_XAXIS,   0, "LookUD" },
    {InputManager::INPUT_PLAYER_LOOKLR, DIMOUSE_YAXIS,   0, "LookLR" },
    {InputManager::INPUT_PLAYER_FIRE,   DIMOUSE_BUTTON0, 0, "Fire"   },
    {InputManager::INPUT_MENU_DEBUG,    DIKEYBOARD_D,    0, "DEBUG" },
    {InputManager::INPUT_MENU_PAUSE,    DIKEYBOARD_ESCAPE,    0, "Pause" }


    /* Actions mapped to keys as well as to virtual controls
    {InputManager::INPUT_PLAYER_ACCELERATE,    DIKEYBOARD_UP,       0, "Accelerate",    },
    {InputManager::INPUT_PLAYER_LEFT_TURN,     DIKEYBOARD_LEFT,     0, "Steer Left",    },
    {InputManager::INPUT_PLAYER_RIGHT_TURN,    DIKEYBOARD_RIGHT,    0, "Steer Right",   },
    {InputManager::INPUT_PLAYER_BRAKE,         DIKEYBOARD_DOWN,     0, "Brake",         },
    {InputManager::INPUT_PLAYER_HORN,          DIKEYBOARD_H,        0, "Horn",          },
    {InputManager::INPUT_PLAYER_THROW_PACKAGE, DIKEYBOARD_LCONTROL, 0, "Throw Package", },
    {InputManager::INPUT_MENU_CONFIGURE_CTRLS, DIKEYBOARD_ESCAPE,   0, "Controller Setup", },*/
  };

  // Menu Actions used by this app
DIACTION gMenuAction[] =
  {
    { InputManager::INPUT_MENU_UD,         DIAXIS_BROWSER_MOVE,       0, TEXT("Up/down") },
    { InputManager::INPUT_MENU_UP,         DIBUTTON_BROWSER_PREVIOUS, 0, TEXT("Up") },
    { InputManager::INPUT_MENU_DOWN,       DIBUTTON_BROWSER_NEXT,     0, TEXT("Down") },
    { InputManager::INPUT_MENU_SELECT,     DIBUTTON_BROWSER_SELECT,   0, TEXT("Select") },
    { InputManager::INPUT_MENU_QUIT,       DIBUTTON_BROWSER_DEVICE,   0, TEXT("Quit menu") },

    // Keyboard input mappings
    { InputManager::INPUT_MENU_DEBUG,      DIKEYBOARD_D,           0, "DEBUG" },
    { InputManager::INPUT_MENU_UP,         DIKEYBOARD_UP,          0, TEXT("Up") },
    { InputManager::INPUT_MENU_DOWN,       DIKEYBOARD_DOWN,        0, TEXT("Down") },
    { InputManager::INPUT_MENU_SELECT,     DIKEYBOARD_SPACE,       0, TEXT("Select") },
    { InputManager::INPUT_MENU_SELECT,     DIKEYBOARD_RETURN,      0, TEXT("Select") },
    { InputManager::INPUT_MENU_SELECT,     DIKEYBOARD_NUMPADENTER, 0, TEXT("Select") },
    {InputManager::INPUT_MENU_PAUSE,    DIKEYBOARD_ESCAPE,    0, "Pause" },

    // Mouse input mappings
    { InputManager::INPUT_MENU_WHEEL,      DIMOUSE_WHEEL,      0, TEXT("Up/down") },
    { InputManager::INPUT_MENU_SELECT,     DIMOUSE_BUTTON0,    0, TEXT("Select") }
  };


#define NUMBER_OF_GAMEACTIONS    (sizeof(gGameAction)/sizeof(DIACTION))
#define NUMBER_OF_MENUACTIONS    (sizeof(gMenuAction)/sizeof(DIACTION))


/*****************************************************************************/
/*                               CONSTRUCTORS                                */  
/*****************************************************************************/

#ifdef _DEBUG
InputManager::InputManager(Logger *pLog, HWND hWnd)
    :mpLog(pLog),
     mhWnd(hWnd) {

  Initialize();
}
#endif

InputManager::InputManager(HWND hWnd)
    :mhWnd(hWnd) {

  Initialize();
}

/*****************************************************************************/
/*                               DESTRUCTOR                                  */  
/*****************************************************************************/

InputManager::~InputManager() {

}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      Initialize                                                           */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*      Creates the Direct3D input device. A list of available devices is    */
/*      polled to be used in the game through the Set/Get Action functions.  */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

//-----------------------------------------------------------------------------
// Name: StaticInputAddDeviceCB()
// Desc: Static callback helper to call into CMyApplication class
//-----------------------------------------------------------------------------
HRESULT CALLBACK StaticInputAddDeviceCB(CInputDeviceManager::DeviceInfo* pDeviceInfo, 
                                        const DIDEVICEINSTANCE* pdidi, 
                                        LPVOID pParam )
{
  InputManager *pIM = (InputManager*) pParam;

    // Setup the deadzone 
    DIPROPDWORD dipdw;
    dipdw.diph.dwSize       = sizeof(DIPROPDWORD);
    dipdw.diph.dwHeaderSize = sizeof(DIPROPHEADER);
    dipdw.diph.dwObj        = 0;
    dipdw.diph.dwHow        = DIPH_DEVICE;
    dipdw.dwData            = 500;
    pDeviceInfo->pdidDevice->SetProperty( DIPROP_DEADZONE, &dipdw.diph );
    
    if( GET_DIDEVICE_TYPE(pdidi->dwDevType) == DI8DEVTYPE_MOUSE )
        pDeviceInfo->pdidDevice->SetCooperativeLevel(pIM->GetHWND(), DISCL_EXCLUSIVE|DISCL_FOREGROUND );


    return S_OK;
}

void InputManager::Initialize() {

HRESULT hr;

  // Setup action format for the actual gameplay
  ZeroMemory(&mDIActionFormat, sizeof(DIACTIONFORMAT));
  mDIActionFormat.dwSize          = sizeof(DIACTIONFORMAT);
  mDIActionFormat.dwActionSize    = sizeof(DIACTION);
  mDIActionFormat.dwDataSize      = NUMBER_OF_GAMEACTIONS * sizeof(DWORD);
  mDIActionFormat.guidActionMap   = sAppGuid;
  mDIActionFormat.dwGenre         = DIVIRTUAL_FIGHTING_FPS; 
  mDIActionFormat.dwNumActions    = NUMBER_OF_GAMEACTIONS;
  mDIActionFormat.rgoAction       = gGameAction;
  mDIActionFormat.lAxisMin        = -100;
  mDIActionFormat.lAxisMax        = 100;
  mDIActionFormat.dwBufferSize    = 16;

  // Setup action format for the in-game menus
  ZeroMemory(&mDIMenuFormat, sizeof(DIACTIONFORMAT));
  mDIMenuFormat.dwSize          = sizeof(DIACTIONFORMAT);
  mDIMenuFormat.dwActionSize    = sizeof(DIACTION);
  mDIMenuFormat.dwDataSize      = NUMBER_OF_MENUACTIONS * sizeof(DWORD);
  mDIMenuFormat.guidActionMap   = sAppGuid;
  mDIMenuFormat.dwGenre         = DIVIRTUAL_BROWSER_CONTROL;
  mDIMenuFormat.dwNumActions    = NUMBER_OF_MENUACTIONS;
  mDIMenuFormat.rgoAction       = gMenuAction;
  mDIMenuFormat.lAxisMin        = -10;
  mDIMenuFormat.lAxisMax        = 10;
  mDIMenuFormat.dwBufferSize    = 16;
  
  hr = Create(mhWnd, 
              NULL, 
              mDIActionFormat, 
              StaticInputAddDeviceCB, 
              this);
  
  if(FAILED(hr)) {

  #ifdef _DEBUG
    LOG(mpLog) << "Call to CInputDeviceManager::Create() failed. Error: " << DXGetErrorDescription8(hr) << "\n\n";
  #endif
    throw "Call to CInputDeviceManager::Create() failed";
  }
}  

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      ConfigureControls                                                    */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*      Displays the Direct Input built-in dialog for configuring controls.  */
/*      The controls are automatically updated by Direct Input.              */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void InputManager::ConfigureControls() {

HRESULT hr;

  hr = ConfigureDevices(mhWnd, NULL, NULL, DICD_EDIT, NULL);

  if(FAILED(hr)) {

  #ifdef _DEBUG
    LOG(mpLog) << "Call to CInputDeviceManager::ConfigureDevices() failed. Error: " << DXGetErrorDescription8(hr) << "\n\n";
  #endif
    throw "Call to CInputDeviceManager::ConfigureDevices() failed";
  }
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      GetInputState                                                        */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*      Returns a bitstate containing the user controls that are in use.     */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

int InputManager::GetInputState(int aiUserInput[], int *piNumUserInput) {

DeviceInfo* deviceInfos;
DWORD numDeviceInfos;
HRESULT hr;

  // Assume there isn't any input
  *piNumUserInput = 0;

  // Get access to the list of semantically-mapped input devices
  hr = GetDevices(&deviceInfos, &numDeviceInfos);

  if(FAILED(hr)) {

  #ifdef _DEBUG
    LOG(mpLog) << "Call to CInputDeviceManager::GetDevices() failed. Error: " << DXGetErrorDescription8(hr) << "\n\n";
  #endif
    throw "Call to CInputDeviceManager::GetDevices() failed";
  }

  // Loop through all devices and check game input
  for(DWORD i = 0; i < numDeviceInfos; i++) {

    LPDIRECTINPUTDEVICE8 pDIDevice = deviceInfos[i].pdidDevice;
    DIDEVICEOBJECTDATA dod[10];
    DWORD numDod = 10;

    hr = pDIDevice->Acquire();
    hr = pDIDevice->Poll();
    hr = pDIDevice->GetDeviceData(sizeof(DIDEVICEOBJECTDATA),
                                  dod, 
                                  &numDod, 
                                  0);
    if(FAILED(hr))
      continue;

    // Finally, pass the input to the main app
    for(DWORD j = 0; j < numDod; j++, *piNumUserInput++) {

      if(dod[j].dwData == 0x80)      
        mabInput[dod[j].uAppData] = true;
      else
        mabInput[dod[j].uAppData] = false;
    }

    for(j = 1; j < 16; j++) {

      if(mabInput[j] == true)
        aiUserInput[j] = j;
    }
  }  

return 0;
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      GetInputState                                                        */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*      Returns a bitstate containing the user controls that are in use.     */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

int InputManager::GetInputState() {

DeviceInfo* deviceInfos;
DWORD numDeviceInfos;
HRESULT hr;

  // Get access to the list of semantically-mapped input devices
  hr = GetDevices(&deviceInfos, &numDeviceInfos);

  if(FAILED(hr)) {

  #ifdef _DEBUG
    LOG(mpLog) << "Call to CInputDeviceManager::GetDevices() failed. Error: " << DXGetErrorDescription8(hr) << "\n\n";
  #endif
    throw "Call to CInputDeviceManager::GetDevices() failed";
  }

  // Loop through all devices and check game input
  for(DWORD i = 0; i < numDeviceInfos; i++) {

    LPDIRECTINPUTDEVICE8 pDIDevice = deviceInfos[i].pdidDevice;
    DIDEVICEOBJECTDATA dod;
    DWORD numDod = 1;

    hr = pDIDevice->Acquire();
    hr = pDIDevice->Poll();
    hr = pDIDevice->GetDeviceData(sizeof(DIDEVICEOBJECTDATA),
                                  &dod, 
                                  &numDod, 
                                  0);
    if(FAILED(hr))
      continue;

    // Finally, pass the input to the main app
    if(numDod > 0) {

      // For the mouse, we want to know about all mouse movements 
      if(dod.uAppData == INPUT_PLAYER_LOOKUD ||
         dod.uAppData == INPUT_PLAYER_LOOKLR) {

        mfAxisState = (float)((int)dod.dwData)/2.0f;
        return dod.uAppData;
      }
      
      // For buttons, we care only when the button is pushed down
      if(dod.dwData == 0x80)      
        return dod.uAppData;
    }
  }  

  return 0;
}






























#if 0
HRESULT               hr; 
LPDIRECTINPUT8        pDIContext; 
 
  hr = DirectInput8Create(mhInstance, 
                          DIRECTINPUT_VERSION, 
                          IID_IDirectInput8, 
                          (void**)&pDIContext, 
                          NULL); 
  if FAILED(hr) { 

  #ifdef _DEBUG
    LOG(mpLog) << "Call to DirectInput8Create() failed. Error: " << DXGetErrorDescription8(hr) << "\n\n";
  #endif
    throw "Call to DirectInput8Create() failed";
  }

  hr = pDIContext->CreateDevice(GUID_SysKeyboard, &pDIDevice, NULL); 

  if FAILED(hr) { 

  #ifdef _DEBUG
    LOG(mpLog) << "Call to CreateDevice() failed. Error: " << DXGetErrorDescription8(hr) << "\n\n";
  #endif
    throw "Call to CreateDevice() failed";
  }

  hr = pDIDevice->SetDataFormat(&c_dfDIKeyboard); 
 
  if FAILED(hr) { 
  
  #ifdef _DEBUG
    LOG(mpLog) << "Call to CreateDevice() failed. Error: " << DXGetErrorDescription8(hr) << "\n\n";
  #endif
    throw "Call to CreateDevice() failed";
  } 

  // Set the cooperative level 
  hr = pDIDevice->SetCooperativeLevel(mhWnd, DISCL_FOREGROUND | DISCL_NONEXCLUSIVE); 
   
  if FAILED(hr) { 
     
  #ifdef _DEBUG
    LOG(mpLog) << "Call to SetCooperativeLevel() failed. Error: " << DXGetErrorDescription8(hr) << "\n\n";
  #endif
    throw "Call to SetCooperativeLevel() failed";
  } 

  pDIDevice->Acquire(); 
#endif
