/*****************************************************************************/
/* File: MidwayMathApp.cpp                                                   */
/*                                                                           */
/* Description:                                                              */
/*                                                                           */
/* Application object for the Midway Math project. Creates subsystem         */
/* objects and intializes them as appropriate. Contains Windows Message      */
/* Handler to pass any recieved keystrokes to related objects.               */
/* Creates a global application class which is passed any Windows' Messages  */
/* recieved from the message callback function.                              */
/*                                                                           */
/* All portions Copyright (c) Austin Krauss, 2003.                           */
/*****************************************************************************/

                           /* MODIFICATION LOG */
/*****************************************************************************/
/*  Date    *  Rev # *  Who     *   Comment                                  */
/*---------------------------------------------------------------------------*/
/* 11/18/03  *   00   *  Austin  * Created.                                  */
/*****************************************************************************/

/*****************************************************************************/
/*                                 HEADERS                                   */  
/*****************************************************************************/

#include "MidwayMathApp.h"
#include "Ground.h"
#include "DIUtil.h"
#include <tokamak.h>

using namespace MidwayMath;

void CollisionCallback (neCollisionInfo &collisionInfo);
/*****************************************************************************/
/*                                CONSTANTS                                  */  
/*****************************************************************************/

const long MidwayMathApp::MIDWAY_STATE_GAME      = 0;
const long MidwayMathApp::MIDWAY_STATE_MENU      = 1;
const long MidwayMathApp::MIDWAY_STATE_PAUSE     = 2;
const long MidwayMathApp::MIDWAY_STATE_GAME_OVER = 3;

const long  MidwayMathApp::MIDWAY_PHYSICS_COUNT    = 16;
const float MidwayMathApp::MIDWAY_PHYSICS_GRAVITY  = -100.0f;

const long MidwayMathApp::MIDWAY_CRATE_COUNT = 15;
const char* const MidwayMathApp::MIDWAY_MODEL_CRATE1 = ".\\Models\\crate1.x";
const char* const MidwayMathApp::MIDWAY_MODEL_CRATE3 = ".\\Models\\crate3.x";
const char* const MidwayMathApp::MIDWAY_MODEL_CRATE4 = ".\\Models\\crate4.x";
const char* const MidwayMathApp::MIDWAY_MODEL_CRATE5 = ".\\Models\\crate5.x";
const char* const MidwayMathApp::MIDWAY_MODEL_GROUND = ".\\Models\\ground.x";
const char* const MidwayMathApp::MIDWAY_MODEL_BALL   = ".\\Models\\ball.x";

const char MidwayMathApp::MIDWAY_MATH_PLUS_OP = '+';
const char MidwayMathApp::MIDWAY_MATH_MINUS_OP = '-';

const long MidwayMathApp::MIDWAY_SOUND_INDEX_GAMEEND     = 0;
const long MidwayMathApp::MIDWAY_SOUND_INDEX_RIGHTANSWER = 1;
const long MidwayMathApp::MIDWAY_SOUND_INDEX_WRONGANSWER = 2;
const long MidwayMathApp::MIDWAY_SOUND_INDEX_BIRD        = 3;
const long MidwayMathApp::MIDWAY_SOUND_INDEX_HIT1        = 4;
const long MidwayMathApp::MIDWAY_SOUND_INDEX_HIT2        = 5;
const long MidwayMathApp::MIDWAY_SOUND_INDEX_HIT3        = 6;
const long MidwayMathApp::MIDWAY_SOUND_INDEX_THUD        = 7;

const char* const MidwayMathApp::MIDWAY_SOUND_GAMEEND      = ".\\Sounds\\gameend.wav";
const char* const MidwayMathApp::MIDWAY_SOUND_RIGHTANSWER  = ".\\Sounds\\rightanswer.wav";
const char* const MidwayMathApp::MIDWAY_SOUND_WRONGANSWER  = ".\\Sounds\\wronganswer.wav";
const char* const MidwayMathApp::MIDWAY_SOUND_BIRD         = ".\\Sounds\\bird.wav"; 
const char* const MidwayMathApp::MIDWAY_SOUND_HIT1         = ".\\Sounds\\hit1.wav"; 
const char* const MidwayMathApp::MIDWAY_SOUND_HIT2         = ".\\Sounds\\hit2.wav"; 
const char* const MidwayMathApp::MIDWAY_SOUND_HIT3         = ".\\Sounds\\hit3.wav"; 
const char* const MidwayMathApp::MIDWAY_SOUND_THUD         = ".\\Sounds\\thud.wav"; 

const char* soundFiles[] = 
    { MidwayMathApp::MIDWAY_SOUND_GAMEEND,
      MidwayMathApp::MIDWAY_SOUND_RIGHTANSWER,  
      MidwayMathApp::MIDWAY_SOUND_WRONGANSWER,
      MidwayMathApp::MIDWAY_SOUND_BIRD,
      MidwayMathApp::MIDWAY_SOUND_HIT1,
      MidwayMathApp::MIDWAY_SOUND_HIT2,
      MidwayMathApp::MIDWAY_SOUND_HIT3,
      MidwayMathApp::MIDWAY_SOUND_THUD
    };         

// Crate X,Y Coordinates
float crateCoords[] = 
    {   0.0f, 22.72f, -64.0f, 22.72f, 60.0f, 22.72f, 123.0f, 22.72f, -127.0f, 22.72f, 
        27.0f, 63.177f, -35.0f, 63.177f, -96.0f, 63.177f, 91.0f, 63.177f,
        59.0f, 103.634f, -6.0f, 103.634f, -64.0f, 103.634f,
        -37.0f, 144.091f, 28.0f, 144.091f,
        -6.0f, 184.432f
    };


/*****************************************************************************/
/*                               CONSTRUCTORS                                */  
/*****************************************************************************/

#ifdef _DEBUG
MidwayMathApp::MidwayMathApp(Logger *pLog, HINSTANCE hInst)
    :mpLog(pLog),
     mhInstance(hInst),
     mbRightAnswer(true) {

  Initialize();
}
#endif

MidwayMathApp::MidwayMathApp(HINSTANCE hInst)
    :mhInstance(hInst), 
     mbRightAnswer(true) {

#ifdef _DEBUG
  mpLog = new Logger("MMApp.log");
#endif    

  Initialize();
}

/*****************************************************************************/
/*                               DESTRUCTOR                                  */  
/*****************************************************************************/

MidwayMathApp::~MidwayMathApp() {

  //Destroy Window
  if(!DestroyWindow(mhWnd)) {

  #ifdef _DEBUG
    LOG(mpLog) << "Call to DestroyWindow failed.\n\n";
  #endif
  }
  
  
  if(!UnregisterClass("MidwayMathApp", mhInstance)) {

  #ifdef _DEBUG
    LOG(mpLog) << "Call to UnregisterClass failed.\n\n";
  #endif
  }

  //Destroy any objects
  delete mpRenderer;
  neSimulator::DestroySimulator(mpPhysics);

  delete mpSoundManager;

#ifdef _DEBUG
  delete mpLog;
#endif
}


/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      Initialize                                                           */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*      Sets up common application level systems for all constructors of this*/
/*      class. Responsible for creating the viewing window, creating helper  */
/*      objects, and loading file resources.                                 */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void MidwayMathApp::Initialize() {

WNDCLASS wc;
bool fullScreen = true;

  if(MessageBox(NULL, 
                "Play in full screen?", 
                "Midway Math", 
                MB_YESNO | MB_ICONQUESTION) 
                == IDNO)
     fullScreen = false;

  //Create the Viewing Window
  wc.lpszClassName = "MidwayMathApp";                             // Class Name
  wc.lpfnWndProc   = WinMessageHandler;                               // Window message handler callback
  wc.style         = CS_DBLCLKS; //| CS_OWNDC | CS_HREDRAW | CS_VREDRAW; // Window style
  wc.hInstance     = mhInstance;                                      // Instance handle
  wc.hIcon         = NULL;                                            // Use default icon when minimized
  wc.hCursor       = LoadCursor(NULL, IDC_ARROW);                     // Windows default arrow
  wc.hbrBackground = (HBRUSH)GetStockObject(BLACK_BRUSH); 
  wc.lpszMenuName  = NULL;                                            // No menu
  wc.cbClsExtra    = 0;                                               // Extra bytes not needed
  wc.cbWndExtra    = 0;                                               // Extra bytes not needed

  if(!RegisterClass(&wc)) {

  #ifdef _DEBUG
    LOG(mpLog) << "Call to RegisterClass failed.\n\n";
  #endif
    throw "Call to RegisterClass failed";
  }

  // Create and show the main window
  mhWnd = CreateWindowEx(WS_EX_DLGMODALFRAME,
                         "MidwayMathApp", 
                         "Midway Math",
                         WS_VISIBLE|WS_POPUP|WS_CAPTION|WS_SYSMENU,// | WS_OVERLAPPEDWINDOW,
                         CW_USEDEFAULT, 
                         CW_USEDEFAULT,
                         800,
                         600,
                         0L, 
                         NULL, 
                         mhInstance, 
                         0L);
   
  if(!mhWnd) {

  #ifdef _DEBUG
    LOG(mpLog) << "Call to CreateWindow returned <null>.\n\n";
  #endif
    throw "Call to CreateWindow failed";
  }
  
  // Set the game state
  mlCurrentState = MIDWAY_STATE_MENU;
  
  // Create the Renderer
  if(fullScreen)
    mpRenderer = new Renderer(mpLog, mhWnd, Renderer::RENDERER_RES_800X600, true); 
  else
    mpRenderer = new Renderer(mpLog, mhWnd, Renderer::RENDERER_RES_640X480, false); 
  
  // Create the physics engine
  InitializePhysics();

  // Initialize the random number generator
  srand((unsigned int) DXUtil_Timer(TIMER_GETELAPSEDTIME));

  // Create the root node of the scene graph
  Ground *ground = new Ground(mpLog, mpRenderer, mpPhysics, MIDWAY_MODEL_GROUND); 
  ground->SetCollisionId(0);
  
  // Set the root node of the Scene Graph
  mpRootNode = ground;  
  mpRenderer->SetRootNode(mpRootNode);

  // Create the camera
  D3DDISPLAYMODE dmode;
  mpRenderer->GetDevice()->GetDisplayMode(&dmode);
  mpCamera = new Camera(mpLog, &dmode, D3DXVECTOR3(0.0f,50.0f,-300.0f), D3DXVECTOR3(0,50,0)); 
  mpRenderer->SetCamera(mpCamera);
  
  // Create the menu UI and menus
  mpUI = new MidwayMathUI(mpLog, mpRenderer);
  InitializeMenus();

  // Load any Resources (levels, models, sounds, etc.)
  InitializeGameObjects();
  InitializeSounds();
  
  //send the matrices to the hardware
  mpRenderer->GetDevice()->SetTransform(D3DTS_PROJECTION, &mpCamera->GetProjMatrix());
  mpRenderer->GetDevice()->SetTransform(D3DTS_VIEW, &mpCamera->GetViewMatrix());
  
  // Create the Input Manager
  mpInputManager = new InputManager(mpLog, mhWnd);
  mpInputManager->UseMenuControls();
  
  // Initialize the application timer
  DXUtil_Timer(TIMER_START);
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      InitializePhysics                                                    */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void MidwayMathApp::InitializePhysics() {
    
neV3 gravity;       // A vector to store the direction and intensity of gravity
neSimulatorSizeInfo sizeInfo;   // SizeInfo stores data about how many objects we are going to model

  // Tell the simulator how many rigid bodies we have
  sizeInfo.rigidBodiesCount = MIDWAY_PHYSICS_COUNT;
  
  // Tell the simulator how many animated bodies we have
  sizeInfo.animatedBodiesCount = 1;
  
  // Tell the simulator how many bodies we have in total
  s32 totalBody = sizeInfo.rigidBodiesCount + sizeInfo.animatedBodiesCount;
  sizeInfo.geometriesCount = totalBody;
  
  /* The overlapped pairs count defines how many bodies it is possible to be in collision
   * at a single time. The SDK states this should be calculated as:
   * bodies * (bodies-1) / 2
   */
  sizeInfo.overlappedPairsCount = totalBody * (totalBody - 1) / 2;
  
  // We're not using any of these so set them all to zero
  sizeInfo.rigidParticleCount     = 0;
  sizeInfo.constraintsCount       = 0;
  sizeInfo.terrainNodesStartCount = 0;

  // Set the gravity
  gravity.Set(0.0f, MIDWAY_PHYSICS_GRAVITY, 0.0f);

  // create the simulator object
  mpPhysics = neSimulator::CreateSimulator(sizeInfo, NULL, &gravity);
  
  
  // Set the collision responses
  mpPhysics->GetCollisionTable()->Set(0, 0, neCollisionTable::RESPONSE_IGNORE);
  mpPhysics->GetCollisionTable()->Set(0, 1, neCollisionTable::RESPONSE_IMPULSE);
  mpPhysics->GetCollisionTable()->Set(0, 2, neCollisionTable::RESPONSE_IMPULSE_CALLBACK);
  mpPhysics->GetCollisionTable()->Set(0, 3, neCollisionTable::RESPONSE_IMPULSE_CALLBACK);
  mpPhysics->GetCollisionTable()->Set(0, 4, neCollisionTable::RESPONSE_IMPULSE);

  mpPhysics->GetCollisionTable()->Set(1, 1, neCollisionTable::RESPONSE_IGNORE);
  mpPhysics->GetCollisionTable()->Set(1, 2, neCollisionTable::RESPONSE_CALLBACK);
  mpPhysics->GetCollisionTable()->Set(1, 3, neCollisionTable::RESPONSE_IMPULSE_CALLBACK);
  mpPhysics->GetCollisionTable()->Set(1, 4, neCollisionTable::RESPONSE_IMPULSE_CALLBACK);
  
  mpPhysics->GetCollisionTable()->Set(2, 3, neCollisionTable::RESPONSE_IMPULSE_CALLBACK);
  mpPhysics->GetCollisionTable()->Set(2, 2, neCollisionTable::RESPONSE_IMPULSE_CALLBACK);

  mpPhysics->SetCollisionCallback(CollisionCallback);
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      InitializeMenus                                                      */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void MidwayMathApp::InitializeMenus() {

  // Create the Main Menu
  MenuItem *p = mpUI->AddMenuItem("Play Game", MidwayMathUI::UI_ID_MAIN_PLAY);
  p->Select();
  
//  mpUI->AddMenuItem(p, "Options",        MidwayMathUI::UI_ID_MAIN_OPTIONS);
  mpUI->AddMenuItem(p, "Exit",         MidwayMathUI::UI_ID_MAIN_EXIT);

  // Set this as the current menu
  mlCurrentMenu = MidwayMathUI::UI_ID_MAIN_PLAY;

//  mpUI->AddMenuItem(p, "Back",           MidwayMathUI::UI_ID_OPTIONS_BACK);

  // Create the in-game Pause Menu
  p = mpUI->AddMenuItem("Resume", MidwayMathUI::UI_ID_GAME_PAUSE);
  p->Select();
  p = mpUI->AddMenuItem(p, "Quit", MidwayMathUI::UI_ID_GAME_QUIT);

  // Create the End Game Menu
  p = mpUI->AddMenuItem("Return to Menu", MidwayMathUI::UI_ID_GAME_QUIT);
  p->Select();
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      InitializeGameObjects                                                */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void MidwayMathApp::InitializeGameObjects() {

ModelNode *models[4];
ModelFileLoader *loader[4];
Crate *crate[MIDWAY_PHYSICS_COUNT-1];
long index = 0;

  // There are four crate models that we need to load
  models[0] = new ModelNode(mpLog, mpRenderer, "Model");
  models[1] = new ModelNode(mpLog, mpRenderer, "Model");
  models[2] = new ModelNode(mpLog, mpRenderer, "Model");
  models[3] = new ModelNode(mpLog, mpRenderer, "Model");

  loader[0] = new ModelFileLoader(mpLog, mpRenderer, MIDWAY_MODEL_CRATE1, models[0]);
  loader[1] = new ModelFileLoader(mpLog, mpRenderer, MIDWAY_MODEL_CRATE3, models[1]);
  loader[2] = new ModelFileLoader(mpLog, mpRenderer, MIDWAY_MODEL_CRATE4, models[2]);
  loader[3] = new ModelFileLoader(mpLog, mpRenderer, MIDWAY_MODEL_CRATE5, models[3]);

  loader[0]->Load();
  loader[1]->Load();
  loader[2]->Load();
  loader[3]->Load();

  // Now create the crates and attach a random model to it
  for(int i = 0; i < MIDWAY_PHYSICS_COUNT-1; i++) {
    
  
    int model = rand()%4;
    
    crate[i] = new Crate(mpLog, mpRenderer, mpPhysics, mpRootNode, models[model]->GetMesh());

    crate[i]->SetMatBuf(models[model]->GetMatBuf());
    crate[i]->SetAdjBuf(models[model]->GetAdjBuf());
    crate[i]->SetNumMaterials(models[model]->GetNumMaterials());
    crate[i]->SetTextures(models[model]->GetTextures());
    
    crate[i]->SetPosition(D3DXVECTOR3(crateCoords[index], crateCoords[index+1], 0.0f));
    crate[i]->SetOnGround(false);
    
    index += 2;
    
    crate[i]->SetCollisionId(2);
    mpRootNode->AttachChild(crate[i]);
  }
  
  // Finally, create the ball object
  mpBall = new Ball(mpLog, mpRenderer, mpPhysics, mpRootNode, MIDWAY_MODEL_BALL);
  mpBall->SetPosition(D3DXVECTOR3(0.0f, 7.84f, -350.0f));
  mpBall->SetCollisionId(1);
  mpRootNode->AttachChild(mpBall); // the ball is the last child on the list

  // I know, I know, this function is full of resource leaks
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      InitializeSounds                                                     */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void MidwayMathApp::InitializeSounds() {

HRESULT hr;

  mpSoundManager = new CSoundManager();
  hr = mpSoundManager->Initialize(mhWnd, DSSCL_PRIORITY, 2, 22050, 16);

  if(FAILED(hr)) {

  #ifdef _DEBUG
    LOG(mpLog) << "Call to CSoundManager::Initialize() failed. Error: " << DXGetErrorDescription8(hr) << "\n\n";
  #endif
    throw "Call to CSoundManager::Initialize() failed";
  }

  for(int i = 0; i < MIDWAY_SOUND_COUNT; i++) {
  
    hr = mpSoundManager->Create(&mpaSounds[i], (char *)soundFiles[i]);

    if(FAILED(hr)) {

    #ifdef _DEBUG
      LOG(mpLog) << "Error loading sound file: " << soundFiles[i] << "Error: " << DXGetErrorDescription8(hr) << "\n\n";
    #endif
      throw "Error loading sound file";
    }
  }
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      CollisionCallback                                                    */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/
extern MidwayMathApp *gpMainApp; // Yes, this is cheating, no, I don't care.
void CollisionCallback (neCollisionInfo &collisionInfo) {

int id1, id2;
    
  if(collisionInfo.typeA != NE_RIGID_BODY && 
     collisionInfo.typeB != NE_RIGID_BODY)
    return;

  neRigidBody *rbA = (neRigidBody *)collisionInfo.bodyA;
  neRigidBody *rbB = (neRigidBody *)collisionInfo.bodyB;

  id1 = rbA->GetCollisionID();
  id2 = rbB->GetCollisionID();

  if((id1 == 1 && id2 == 3) || (id1 == 3 && id2 == 1)) {

    if(id1 == 3) gpMainApp->HandleRightAnswer((Crate*)rbA->GetUserData()); 
    else if(id2 == 3) gpMainApp->HandleRightAnswer((Crate*)rbB->GetUserData()); 
  }

  else if((id1 == 1 && id2 == 2) || (id1 == 2 && id2 == 1)) {

    if(id1 == 2) gpMainApp->HandleWrongAnswer((Crate*)rbA->GetUserData()); 
    else if(id2 == 2) gpMainApp->HandleWrongAnswer((Crate*)rbB->GetUserData()); 
  }

  else if((id1 == 0 && id2 == 2) || (id1 == 2 && id2 == 0)) {

    if(id1 == 2) gpMainApp->HandleCollapse((Crate*)rbA->GetUserData()); 
    else if(id2 == 2) gpMainApp->HandleCollapse((Crate*)rbB->GetUserData()); 
  }

  else if((id1 == 0 && id2 == 3) || (id1 == 3 && id2 == 0)) {

    if(id1 == 3) gpMainApp->HandleCollapse((Crate*)rbA->GetUserData()); 
    else if(id2 == 3) gpMainApp->HandleCollapse((Crate*)rbB->GetUserData()); 
  }

  else if((id1 == 2 && id2 == 3) || (id1 == 3 && id2 == 2) || (id1 == 2 && id2 == 2)) {

    gpMainApp->HandleCratesCollide();
  }
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      MessageHandler                                                       */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*      hWnd                                                                 */
/*          Handle to the current window.                                    */
/*                                                                           */
/*      msg                                                                  */
/*          Message identifier.                                              */
/*                                                                           */
/*      wParam                                                               */
/*          First message parameter.                                         */
/*                                                                           */
/*      lParam                                                               */
/*          Second message parameter.                                        */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*      Message handler function called from the WinMain module. Process     */
/*      Windows messages by calling appropriate application routines.        */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*      LRESULT                                                              */
/*          Depends on the message sent.                                     */
/*                                                                           */
/*****************************************************************************/

LRESULT MidwayMathApp::MessageHandler(HWND hWnd, 
                                          UINT msg, 
                                          WPARAM wParam, 
                                          LPARAM lParam) {

  switch (msg) {

    case WM_DESTROY: 
    case WM_CLOSE:
      PostQuitMessage(0); 
      return 0;
    default:
      return DefWindowProc(hWnd, msg, wParam, lParam); // Let Windows handle it
  }
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      Run                                                                  */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*      Starts the main event loop for the game. Does not return until the   */
/*      user elects to quit the game completely.                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void MidwayMathApp::Run() {

MSG msg;         

  // Enter main event loop
  while(true) {
    
    // Look for messages
    if(PeekMessage(&msg, NULL, 0, 0, PM_NOREMOVE)) {
      
      if(!GetMessage(&msg, NULL, 0, 0)) 
        // WM_QUIT was posted, so exit
        return;

      // Translate and dispatch the message
      TranslateMessage(&msg); 
      DispatchMessage(&msg);
    }
    
    // main game loop
    else {

      // Read player input
      ReadPlayerInput();
      
      // Are we showing the menu?
      if(mlCurrentState == MIDWAY_STATE_MENU) {
      
        mpRenderer->BeginRenderMenu();
        mpUI->DrawMenu(mlCurrentMenu);
        mpRenderer->EndRenderMenu();
      }
      
      // Are we paused?
      if(mlCurrentState == MIDWAY_STATE_PAUSE) {
      
        // And render our scene
        mpRenderer->GetDevice()->BeginScene();
        mpRenderer->Render();
        mpUI->DrawMenu(MidwayMathUI::UI_ID_GAME_PAUSE);
        mpRenderer->GetDevice()->EndScene();
        mpRenderer->GetDevice()->Present(NULL, NULL, NULL, NULL);
      }
      
      // Is the game over?
      if(mlCurrentState == MIDWAY_STATE_GAME_OVER) {

        if(mbEndSound) {
          DSUtil_PlaySound(mpaSounds[MIDWAY_SOUND_INDEX_GAMEEND]);
          mbEndSound = false;
        }

        mpRenderer->GetDevice()->BeginScene();
        mpRenderer->Render();
        mpUI->DrawMenu(MidwayMathUI::UI_ID_GAME_QUIT);
        mpRenderer->GetDevice()->EndScene();
        mpRenderer->GetDevice()->Present(NULL, NULL, NULL, NULL);
      }
      
      // Are we playing the game?
      if(mlCurrentState == MIDWAY_STATE_GAME) {
       
        if(rand()%100000 <= 1)
          DSUtil_PlaySound(mpaSounds[MIDWAY_SOUND_INDEX_BIRD]);
        
        if(GetRightAnswer()) {
      
          int numCrates = mpRootNode->GetNumberOfChildren()-1;
          bool cratesIdle = true;
          int cratesOnGround = 0;
          for(int i = 0; i < numCrates; i++) { 
            Crate *crate = dynamic_cast<Crate*>(mpRootNode->GetChild(i));
              if(!crate->IsIdle()) cratesIdle = false;
              if(crate->GetOnGround()) cratesOnGround++;
            }
          
          if(cratesOnGround >= numCrates-1) {
            HandleGameOver();
            continue;
          }

          // Create a problem
          if(cratesIdle) {
            miRightAnswers++;
            CreateMathProblem();
          }
        }

        // Update the physics
        UpdatePhysics();

        // Update our scene graph
        mpRootNode->Update();

        // And render our scene
        mpRenderer->GetDevice()->BeginScene();
        mpRenderer->Render();
        mpUI->DrawHUD();
        mpRenderer->GetDevice()->EndScene();
        mpRenderer->GetDevice()->Present(NULL, NULL, NULL, NULL);
      }
    }
  }
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      UpdatePhysics                                                        */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void MidwayMathApp::UpdatePhysics() { 

static float lastTime = 0.0f;
float fElapsed;

  // Find out how much time has elapsed since we were last called
  fElapsed = DXUtil_Timer(TIMER_GETELAPSEDTIME);

  /* In order to keep passing a reasonably stable value into the simulator's Advance
   * method, we'll try to stop occasional variations in the elapsed time from occurring.
   * We'll do this by preventing the elapsed time from being more than 20% greater or
   *less than the previous elapsed time.
   */
  if (lastTime != 0) {

    if (fElapsed > lastTime * 1.2f) fElapsed = lastTime * 1.2f;
    if (fElapsed < lastTime * 0.8f) fElapsed = lastTime * 0.8f;
  }

  /* To avoid particularly large spikes, we'll stop the elapsed time from exceeding
   * 1/45th of a second. Values larger than this can lead to an unstable simulation.
   */
  if (fElapsed > 1.0f / 45.0f) fElapsed = 1.0f / 45.0f;

  // Store the elapsed time so that we can use it the next time around
  lastTime = fElapsed;

  /* Tell the Tokamak simulator to advance all its calculations by one frame.
   * Note that we pass the elapsed time here so that the simulation speed
   * is constant regardless of the refresh frequency, frame rate, etc.
   */
  mpPhysics->Advance(fElapsed,4);
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      ReadPlayerInput                                                      */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void MidwayMathApp::ReadPlayerInput() { 

CInputDeviceManager::DeviceInfo *deviceInfos;
DWORD numDeviceInfos;
float fAxisState;
HRESULT hr;

  // Get access to the list of semantically-mapped input devices
  hr = mpInputManager->GetDevices(&deviceInfos, &numDeviceInfos);

  if(FAILED(hr)) {

  #ifdef _DEBUG
    LOG(mpLog) << "Call to CInputDeviceManager::GetDevices() failed. Error: " << DXGetErrorDescription8(hr) << "\n\n";
  #endif
    throw "Call to CInputDeviceManager::GetDevices() failed";
  }

  // Loop through all devices and check game input
  for(DWORD i = 0; i < numDeviceInfos; i++) {

    LPDIRECTINPUTDEVICE8 pDIDevice = deviceInfos[i].pdidDevice;
    DIDEVICEOBJECTDATA dod[10];
    DWORD numDod = 10;

    hr = pDIDevice->Acquire();
    hr = pDIDevice->Poll();
    hr = pDIDevice->GetDeviceData(sizeof(DIDEVICEOBJECTDATA),
                                  dod, 
                                  &numDod, 
                                  0);
    if(FAILED(hr))
      continue;

    for(DWORD j = 0; j < numDod; j++) {
      
      // For the mouse, we want to know about all mouse movements 
      if(dod[j].uAppData == InputManager::INPUT_PLAYER_LOOKUD ||
         dod[j].uAppData == InputManager::INPUT_PLAYER_LOOKLR) {

        fAxisState = (float)((int)dod[j].dwData)/2.0f;
      }
      
      // For buttons, we care only when the button is pushed down
      else if(dod[j].dwData != 0x80)      
        continue;
    
      switch(dod[j].uAppData) {

        case InputManager::INPUT_PLAYER_LOOKUD:  
        case InputManager::INPUT_PLAYER_LOOKLR:  
        case InputManager::INPUT_PLAYER_FIRE:     HandleMouseMessage(dod[j].uAppData, fAxisState);     break;
        case InputManager::INPUT_MENU_PAUSE:      HandleMenuPause();  break;
        case InputManager::INPUT_MENU_MOUSE_MOVE: LOG(mpLog) << "Got INPUT_MENU_MOUSE_MOVE.\n\n";  break;
        case InputManager::INPUT_MENU_UD:         LOG(mpLog) << "Got INPUT_MENU_UD.\n\n";         break;
        case InputManager::INPUT_MENU_UP:         mpUI->MenuUp(mlCurrentMenu);                        break;
        case InputManager::INPUT_MENU_DOWN:       mpUI->MenuDown(mlCurrentMenu);                      break;
        case InputManager::INPUT_MENU_SELECT:     HandleMenuSelect();                             break; 
        case InputManager::INPUT_MENU_QUIT:       LOG(mpLog) << "Got INPUT_MENU_QUIT.\n\n";           break;    
        case InputManager::INPUT_MENU_WHEEL:      LOG(mpLog) << "Got INPUT_MENU_WHEEL.\n\n";      break; 
        case InputManager::INPUT_MENU_DEBUG:      mpRenderer->SetDebug(!mpRenderer->GetDebug());   break;
      }
    }
  }
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      HandleMenuSelect                                                     */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void MidwayMathApp::HandleMenuSelect() {

MenuItem *m = mpUI->GetSelected(mlCurrentMenu);
long id = m->GetId();

  if(id == MidwayMathUI::UI_ID_MAIN_PLAY) {
    ResetGameObjects();
    mpInputManager->UseGameControls();
    mlCurrentState = MIDWAY_STATE_GAME; 
  }

  if(id == MidwayMathUI::UI_ID_GAME_QUIT) {
    mpInputManager->UseMenuControls();
    mlCurrentState = MIDWAY_STATE_MENU; 
    mlCurrentMenu = MidwayMathUI::UI_ID_MAIN_PLAY;
  }

  else if(id == MidwayMathUI::UI_ID_MAIN_OPTIONS) mlCurrentMenu = MidwayMathUI::UI_ID_OPTIONS_SOUND;
  else if(id == MidwayMathUI::UI_ID_MAIN_INTRO);
  else if(id == MidwayMathUI::UI_ID_OPTIONS_BACK) mlCurrentMenu = MidwayMathUI::UI_ID_MAIN_PLAY;
  else if(id == MidwayMathUI::UI_ID_GAME_PAUSE) HandleMenuPause();
  else if(id == MidwayMathUI::UI_ID_MAIN_EXIT) PostQuitMessage(0);
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      HandleRightAnswer                                                    */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void MidwayMathApp::HandleRightAnswer(Crate *crate) {

Crate *c;

  for(int i = 0; i < MIDWAY_CRATE_COUNT; i++) {

    c = dynamic_cast<Crate*>(mpRootNode->GetChild(i));
    if(c != crate) 
      c->SetNumber(-1);
  }
  
  if(rand()%100 <= 33)
    { DSUtil_PlaySound(mpaSounds[MIDWAY_SOUND_INDEX_HIT1]); }
  else if(rand()%100 <= 66)
    { DSUtil_PlaySound(mpaSounds[MIDWAY_SOUND_INDEX_HIT2]); }
  else
    { DSUtil_PlaySound(mpaSounds[MIDWAY_SOUND_INDEX_HIT3]); }

  if(rand()%100 <= 20)
    DSUtil_PlaySound(mpaSounds[MIDWAY_SOUND_INDEX_RIGHTANSWER]);

  SetRightAnswer(true);
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      HandleWrongAnswer                                                    */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void MidwayMathApp::HandleWrongAnswer(Crate *crate) {

  if(crate->GetNumber() != -1) {  
    DSUtil_PlaySound(mpaSounds[MIDWAY_SOUND_INDEX_WRONGANSWER]);
    miWrongAnswers++;
    crate->SetNumber(-1);
  }
  
  mpBall->SetPosition(D3DXVECTOR3(0.0f, 7.84f, -350.0f));
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      HandleWrongAnswer                                                    */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void MidwayMathApp::HandleCollapse(Crate *crate) {

//  if(!crate->GetOnGround())
//    DSUtil_PlaySound(mpaSounds[MIDWAY_SOUND_INDEX_THUD]);
  crate->SetOnGround(true);
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      HandleCratesCollide                                                  */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void MidwayMathApp::HandleCratesCollide() {

}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      HandleMenuSelect                                                     */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void MidwayMathApp::HandleMouseMessage(int mouseAxis, float axis) {

D3DXVECTOR3 look;

  if(mouseAxis == InputManager::INPUT_PLAYER_FIRE) {

    mpBall->SetPosition(mpCamera->GetEyePt());
    miNumThrows++;
    mpBall->Throw(mpCamera->GetViewDir());
    return;
  }

  look = mpCamera->GetLookatPt();

  if(mouseAxis == InputManager::INPUT_PLAYER_LOOKUD) 
    look.y += axis;
  else 
    look.x += axis;
  
  if(look.y > 200.0f) look.y = 200.0f;
  if(look.y < -5.0f)  look.y = -5.0f;
  if(look.x > 220.0f) look.x = 220.0f;
  if(look.x < -220.0f) look.x = -220.0f;
  
  mpCamera->SetLookatPt(look);
  mpRenderer->GetDevice()->SetTransform(D3DTS_VIEW, &mpCamera->GetViewMatrix());
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      HandleMenuPause                                                      */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void MidwayMathApp::HandleMenuPause() {

  if(mlCurrentState == MIDWAY_STATE_GAME) {
    mpInputManager->UseMenuControls();
    mlCurrentState = MIDWAY_STATE_PAUSE;
    mlCurrentMenu = MidwayMathUI::UI_ID_GAME_PAUSE;
  }
  else if(mlCurrentState == MIDWAY_STATE_PAUSE) {
    mpInputManager->UseGameControls();
    mlCurrentState = MIDWAY_STATE_GAME; 
    mlCurrentMenu = MidwayMathUI::UI_ID_MAIN_PLAY;
  }
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      CreateMathProblem                                                    */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

bool MidwayMathApp::CreateMathProblem() {

int numCrates = 0;
Crate *crates[MIDWAY_CRATE_COUNT];
int numAnswers;
int *answers;

  /* Search for crates that haven't hit the ground */
  for(int i = 0; i < MIDWAY_CRATE_COUNT; i++) {

    Crate *crate = dynamic_cast<Crate*>(mpRootNode->GetChild(i));
    if(!crate->GetOnGround()) 
      crates[numCrates++] = crate;
  }

  if(numCrates == 0 || numCrates == 1)
    return false;

  numAnswers = (numCrates/2)+1;
  answers = new int[numAnswers];

char cans[32];
int op1, op2;
char coperator;

  op1 = rand()%11;
  op2 = rand()%11;

  if(rand()%2 == 0) {
    coperator = MIDWAY_MATH_PLUS_OP;
    answers[0] = op1 + op2;
  }
  else {

    int t;
    if(op1 < op2) {
      t = op1;
      op1 = op2;
      op2 = t;
    }
  
    coperator = MIDWAY_MATH_MINUS_OP;
    answers[0] = op1 - op2;
  }

  sprintf(cans, "%d %c %d =", op1, coperator, op2); 
  mpUI->SetProblem(cans);

  int curAns = 1;
  while(curAns < numAnswers) {

    int a = rand()%21;
    bool dup = false;

    for(int i = 0; i < curAns; i++) {
      if(answers[i] == a) {
        dup = true;
        break;
      }
    }

    if(dup)
      continue;
    
    answers[curAns] = a;
    curAns++;
  }

  /* Fill all the crates with -1 */
  for(i = 0; i < MIDWAY_CRATE_COUNT; i++) {

    Crate *crate = dynamic_cast<Crate*>(mpRootNode->GetChild(i));
    crate->SetNumber(-1);
    crate->SetCollisionId(2);
  }
        
  numAnswers--;
  Crate *crate;
  while(numAnswers >= 0) {

    crate = crates[rand()%numCrates];
    if(crate->GetNumber() == -1) {
      
      crate->SetNumber(answers[numAnswers]);
      numAnswers--;
    }
  }

  crate->SetCollisionId(3);
  SetRightAnswer(false);
  delete [] answers;
  return true;
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      ResetGameObjects                                                     */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void MidwayMathApp::ResetGameObjects() {

int index = 0;

  for(int i = 0; i < MIDWAY_PHYSICS_COUNT-1; i++) {
    
    Crate *crate = dynamic_cast<Crate*>(mpRootNode->GetChild(i));
    crate->SetPosition(D3DXVECTOR3(crateCoords[index], crateCoords[index+1], 0.0f));
    crate->SetOnGround(false);
    crate->SetCollisionId(2);
    
    index += 2;
  }
  
  miRightAnswers = 0;
  miWrongAnswers = 0;
  miNumThrows = 0;
  
  
  mpBall->SetPosition(D3DXVECTOR3(0.0f, 7.84f, -350.0f));
  CreateMathProblem();
  mbEndSound = true;
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      HandleGameOver                                                       */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void MidwayMathApp::HandleGameOver() {

char temp1[128];
char temp2[128];
char temp3[128];
  
  sprintf(temp1, "Number of Right Answers: %d", miRightAnswers);
  sprintf(temp2, "Number of Wrong Answers: %d", miWrongAnswers);
  sprintf(temp3, "Number of Balls Thrown: %d", miNumThrows); 

  mpUI->SetStat1(temp1);
  mpUI->SetStat2(temp2);
  mpUI->SetStat3(temp3);
  mpInputManager->UseMenuControls();
  mlCurrentState = MIDWAY_STATE_GAME_OVER;
  mlCurrentMenu = MidwayMathUI::UI_ID_GAME_QUIT;
}