/*****************************************************************************/
/* File: MidwayMathUI.cpp                                                    */
/*                                                                           */
/* Description:                                                              */
/*                                                                           */
/* All portions Copyright (c) Austin Krauss, 2003.                           */
/*****************************************************************************/

                           /* MODIFICATION LOG */
/*****************************************************************************/
/*  Date    *  Rev # *  Who     *   Comment                                  */
/*---------------------------------------------------------------------------*/
/* 11/20/03 *   00   *  Austin  * Created.                                   */
/*****************************************************************************/

/*****************************************************************************/
/*                                 HEADERS                                   */  
/*****************************************************************************/

#include "MidwayMathUI.h"
#include "Dxerr8.h"

using namespace MidwayMath;

/*****************************************************************************/
/*                                CONSTANTS                                  */  
/*****************************************************************************/

const long MidwayMathUI::UI_ID_MAIN_PLAY      = 0;
const long MidwayMathUI::UI_ID_MAIN_OPTIONS   = 1;
const long MidwayMathUI::UI_ID_MAIN_INTRO     = 2;
const long MidwayMathUI::UI_ID_MAIN_EXIT      = 3;
const long MidwayMathUI::UI_ID_OPTIONS_SOUND  = 4;
const long MidwayMathUI::UI_ID_OPTIONS_VIDEO  = 5;
const long MidwayMathUI::UI_ID_OPTIONS_BACK   = 6;
const long MidwayMathUI::UI_ID_GAME_PAUSE     = 7;
const long MidwayMathUI::UI_ID_GAME_QUIT      = 8;

const char* const MidwayMathUI::UI_TEXTURE_MAIN_MENU = "./Textures/MainMenu.bmp";

/*****************************************************************************/
/*                               CONSTRUCTORS                                */  
/*****************************************************************************/

#ifdef _DEBUG
MidwayMathUI::MidwayMathUI(Logger *l, Renderer *r) 
  :mfTitlePosition(65.0f) {

  Initialize(l, r);

  // create fonts
  CreateFonts();

  // create the background images
  CreateImages();
}
#endif

MidwayMathUI::MidwayMathUI(Renderer *r) 
  :mfTitlePosition(65.0f) {

  Initialize(r);

  // create fonts
  CreateFonts();

  // create the background images
  CreateImages();
}

/*****************************************************************************/
/*                               DESTRUCTOR                                  */  
/*****************************************************************************/

MidwayMathUI::~MidwayMathUI() {

  SAFE_RELEASE(mpTitleText);
  SAFE_RELEASE(mpViewport);
  delete(mpFont);
}


/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      DrawMenu                                                             */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void MidwayMathUI::DrawMenu(long id) {

  MenuItem *menu = FindSubMenu(id);

  D3DXMATRIX matView, matViewSaved; 
  D3DXMATRIX matProj, matProjSaved; 
  D3DXMATRIX matWorld, matTemp;

  float menuPos = -0.5;

  if(!menu) {
  
  #ifdef _DEBUG
    LOG(mpLog) << "Could not find menu id: " << id << " Unable to render menu.\n\n";
  #endif
  
    return;
  }

  // Save current matrices
  mpRenderer->GetDevice()->GetTransform(D3DTS_VIEW,       &matViewSaved);
  mpRenderer->GetDevice()->GetTransform(D3DTS_PROJECTION, &matProjSaved);

  // Setup new view and proj matrices for head-on viewing
  D3DXMatrixLookAtLH(&matView, &D3DXVECTOR3(0.0f,0.0f,-30.0f),
                               &D3DXVECTOR3(0.0f,0.0f,0.0f),
                               &D3DXVECTOR3(0.0f,1.0f,0.0f));
  D3DXMatrixPerspectiveFovLH(&matProj, D3DX_PI/4, 1.0f, 1.0f, 100.0f);
  mpRenderer->GetDevice()->SetTransform(D3DTS_VIEW,       &matView);
  mpRenderer->GetDevice()->SetTransform(D3DTS_PROJECTION, &matProj);

  if(id == UI_ID_GAME_PAUSE) {
    DrawPauseMenu();
  }

  if(id == UI_ID_GAME_QUIT) {
    DrawQuitMenu();
    menuPos = -9.0f;
  }
  
  // special considerations for the main menu
  if(id == UI_ID_MAIN_PLAY ||
     id == UI_ID_OPTIONS_SOUND) {
    
    LPDIRECT3DSURFACE8 backbuffer;

    // Display the image
    mpRenderer->GetDevice()->GetBackBuffer(0, D3DBACKBUFFER_TYPE_MONO, &backbuffer);
    mpRenderer->GetDevice()->CopyRects(mpBackgroundImage, NULL, 0, backbuffer, NULL);
    backbuffer->Release();

    mpRenderer->GetDevice()->SetRenderState(D3DRS_LIGHTING, TRUE);
    mpRenderer->GetDevice()->SetRenderState(D3DRS_AMBIENT, 0x80808080);
    
    // world matrix for title text
    D3DXMatrixScaling(&matWorld, 3.0f, 3.0f, 3.0f);
    matWorld._41 = -8.0f;
    matWorld._42 = 8.0f;
    matWorld._43 = mfTitlePosition;
    mpRenderer->GetDevice()->SetTransform(D3DTS_WORLD, &matWorld);
    
    mfTitlePosition -= .5f;
    if(mfTitlePosition <= 0.0f) 
      mfTitlePosition = 0.0f;
    
    mpRenderer->GetDevice()->SetMaterial(&mTitleMaterial);
    mpTitleText->DrawSubset(0);
  }

  mpRenderer->GetDevice()->SetRenderState(D3DRS_LIGHTING, TRUE);
  mpRenderer->GetDevice()->SetRenderState(D3DRS_AMBIENT,  0xFFFFFFFF);

  // Render the top level menu item
  if(menu->IsSelected()) {

    D3DXMatrixScaling(&matWorld, 0.4f, 0.4f, 0.4f);
    D3DXMatrixRotationY(&matTemp, (D3DX_PI/3)*sinf(timeGetTime()/200.0f));
    matWorld *= matTemp;

    mpRenderer->GetDevice()->SetMaterial(&mSelectedMaterial);
  }
  else {

    D3DXMatrixScaling(&matWorld, 0.3f, 0.3f, 0.3f);
    mpRenderer->GetDevice()->SetMaterial(&mNormalMaterial);
  }
  
  matWorld._42 = menuPos;
  menuPos -= 3.0f;
  mpRenderer->GetDevice()->SetTransform(D3DTS_WORLD, &matWorld);

  mpFont->Render3DText((char*) menu->GetLabel().c_str(), 
                       D3DFONT_CENTERED|D3DFONT_TWOSIDED);
  
  
  // Loop through and render all children menu items
  for(int i = 0; i < menu->GetNumberOfChildren(); i++)  {

    // Give a different effect for selected items
    if(menu->GetChild(i)->IsSelected()) {
        
      D3DXMatrixScaling(&matWorld, 0.4f, 0.4f, 0.4f);
      D3DXMatrixRotationY(&matTemp, (D3DX_PI/3)*sinf(timeGetTime()/200.0f));
      matWorld *= matTemp;

      mpRenderer->GetDevice()->SetMaterial(&mSelectedMaterial);
    }
    else {

      D3DXMatrixScaling(&matWorld, 0.3f, 0.3f, 0.3f);
      mpRenderer->GetDevice()->SetMaterial(&mNormalMaterial);
    }
    
    // Translate the menuitem into place
    matWorld._42 = menuPos;
    menuPos -= 3.0f;
    mpRenderer->GetDevice()->SetTransform(D3DTS_WORLD, &matWorld);

    // Render the menuitem's label
    mpFont->Render3DText((char*) menu->GetChild(i)->GetLabel().c_str(), 
                         D3DFONT_CENTERED|D3DFONT_TWOSIDED);
  }

  // Restore matrices
  mpRenderer->GetDevice()->SetTransform(D3DTS_VIEW,       &matViewSaved);
  mpRenderer->GetDevice()->SetTransform(D3DTS_PROJECTION, &matProjSaved);
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      MenuDown                                                             */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void MidwayMathUI::MenuDown(long id) {

  MenuItem *menu = FindSubMenu(id);

  if(!menu) {
  
  #ifdef _DEBUG
    LOG(mpLog) << "Could not find menu id: " << id << " Unable to move menu down.\n\n";
  #endif
  
    return;
  }

  // check the top level menu item
  if(menu->IsSelected()) {
    menu->Deselect();
    menu->GetChild(0)->Select();
    return;
  }

  // check the children
  for(int i = 0; i < menu->GetNumberOfChildren(); i++)  {

    if(menu->GetChild(i)->IsSelected()) {

      menu->GetChild(i)->Deselect(); 
      
      // Is this the last menu item?
      if(i+1 == menu->GetNumberOfChildren()) {
        menu->Select();
        return;
      } 
    
      menu->GetChild(i+1)->Select();
      return;
    }
  }
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      MenuUp                                                               */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void MidwayMathUI::MenuUp(long id) {

  MenuItem *menu = FindSubMenu(id);

  if(!menu) {
  
  #ifdef _DEBUG
    LOG(mpLog) << "Could not find menu id: " << id << " Unable to move menu down.\n\n";
  #endif
  
    return;
  }

  // check the top level menu item
  if(menu->IsSelected()) {
    menu->Deselect();
    menu->GetChild(menu->GetNumberOfChildren()-1)->Select();
    return;
  }

  // check the children
  for(int i = 0; i < menu->GetNumberOfChildren(); i++)  {

    if(menu->GetChild(i)->IsSelected()) {

      menu->GetChild(i)->Deselect(); 
      
      // Is this the first menu item?
      if(i == 0) {
        menu->Select();
        return;
      } 
    
      menu->GetChild(i-1)->Select();
      return;
    }
  }
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      GetSelected                                                          */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

MenuItem* MidwayMathUI::GetSelected(long id) {

  MenuItem *menu = FindSubMenu(id);

  if(!menu) {
  
  #ifdef _DEBUG
    LOG(mpLog) << "Could not find menu id: " << id << " Unable to get selected menu.\n\n";
  #endif
  
    return NULL;
  }

  // check the top level menu item
  if(menu->IsSelected()) {
    return menu;
  }

  // check the children
  for(int i = 0; i < menu->GetNumberOfChildren(); i++)  {

    if(menu->GetChild(i)->IsSelected()) 
      return menu->GetChild(i);
  }

  return NULL;
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      DrawHUD                                                              */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void MidwayMathUI::DrawHUD() {

D3DXMATRIX matView, matViewSaved; 
D3DXMATRIX matProj, matProjSaved; 
D3DXMATRIX matWorld;

  // Save current matrices
  mpRenderer->GetDevice()->GetTransform(D3DTS_VIEW,       &matViewSaved);
  mpRenderer->GetDevice()->GetTransform(D3DTS_PROJECTION, &matProjSaved);

  // Setup new view and proj matrices for head-on viewing
  D3DXMatrixLookAtLH(&matView, &D3DXVECTOR3(0.0f,0.0f,-30.0f),
                               &D3DXVECTOR3(0.0f,0.0f,0.0f),
                               &D3DXVECTOR3(0.0f,1.0f,0.0f));
  D3DXMatrixPerspectiveFovLH(&matProj, D3DX_PI/4, 1.0f, 1.0f, 100.0f);
  mpRenderer->GetDevice()->SetTransform(D3DTS_VIEW,       &matView);
  mpRenderer->GetDevice()->SetTransform(D3DTS_PROJECTION, &matProj);


  D3DXMatrixScaling(&matWorld, 0.5f, 0.5f, 0.5f);
  matWorld._42 = -9.0f;
  mpRenderer->GetDevice()->SetTransform(D3DTS_WORLD, &matWorld);

  mpRenderer->GetDevice()->SetMaterial(&mGameFontMaterial);
  mpFont->Render3DText((char *)mProblem.c_str(), D3DFONT_CENTERED|D3DFONT_TWOSIDED);
  
  // Restore matrices
  mpRenderer->GetDevice()->SetTransform(D3DTS_VIEW,       &matViewSaved);
  mpRenderer->GetDevice()->SetTransform(D3DTS_PROJECTION, &matProjSaved);
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      DrawPauseMenu                                                        */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void MidwayMathUI::DrawPauseMenu() {

  DarkenScene(0.3f);
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      DrawQuitMenu                                                         */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void MidwayMathUI::DrawQuitMenu() {

D3DXMATRIX matWorld;
D3DMATERIAL8 mat;

D3DUtil_InitMaterial(mat, 0.0f, 1.0f, 0.0f, 1.0f);

  DarkenScene(0.45f);

  D3DXMatrixScaling(&matWorld, 0.5f, 0.5f, 0.5f);
  matWorld._42 = 7.0f;
  mpRenderer->GetDevice()->SetTransform(D3DTS_WORLD, &matWorld);
  mpRenderer->GetDevice()->SetMaterial(&mat);
  mpFont->Render3DText("Congratulations!!!", D3DFONT_CENTERED|D3DFONT_TWOSIDED);
  
  D3DXMatrixScaling(&matWorld, 0.3f, 0.3f, 0.3f);
  matWorld._42 = 3.0f;
  mpRenderer->GetDevice()->SetTransform(D3DTS_WORLD, &matWorld);

  mpFont->Render3DText((char *)mStats1.c_str(), D3DFONT_CENTERED|D3DFONT_TWOSIDED);

  matWorld._42 = 0.0f;
  mpRenderer->GetDevice()->SetTransform(D3DTS_WORLD, &matWorld);
  mpFont->Render3DText((char *)mStats2.c_str(), D3DFONT_CENTERED|D3DFONT_TWOSIDED);

  matWorld._42 = -3.0f;
  mpRenderer->GetDevice()->SetTransform(D3DTS_WORLD, &matWorld);
  mpFont->Render3DText((char *)mStats3.c_str(), D3DFONT_CENTERED|D3DFONT_TWOSIDED);
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      DarkenScene                                                          */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

// Adapted from donuts.cpp from the DX8.1 SDK
void MidwayMathUI::DarkenScene(float amount) {

  // Setup a dark square to cover the scene
  DWORD dwAlpha = (amount<1.0f) ? ((DWORD)(255*amount))<<24L : 0xff000000;
  
  SCREENVERTEX* v;
  mpViewport->Lock( 0, 0, (BYTE**)&v, 0 );
  v[0].color = v[1].color = v[2].color = v[3].color = dwAlpha;
  mpViewport->Unlock();

  // Set renderstates
  mpRenderer->GetDevice()->SetRenderState( D3DRS_ALPHABLENDENABLE, TRUE );
  mpRenderer->GetDevice()->SetRenderState( D3DRS_SRCBLEND,  D3DBLEND_SRCALPHA );
  mpRenderer->GetDevice()->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA );
  mpRenderer->GetDevice()->SetTexture( 0, NULL );

  // Draw a big, gray square
  mpRenderer->GetDevice()->SetVertexShader((D3DFVF_XYZRHW|D3DFVF_DIFFUSE));
  mpRenderer->GetDevice()->SetStreamSource(0, mpViewport, sizeof(SCREENVERTEX));
  mpRenderer->GetDevice()->DrawPrimitive(D3DPT_TRIANGLESTRIP,0, 2);

  // Restore states
  mpRenderer->GetDevice()->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
  mpRenderer->GetDevice()->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
  mpRenderer->GetDevice()->SetTextureStageState(0, D3DTSS_COLOROP,   D3DTOP_MODULATE);
  mpRenderer->GetDevice()->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
  mpRenderer->GetDevice()->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
  
  mpRenderer->GetDevice()->SetRenderState(D3DRS_ZENABLE, TRUE);
  mpRenderer->GetDevice()->SetRenderState(D3DRS_DITHERENABLE, TRUE);
  mpRenderer->GetDevice()->SetRenderState(D3DRS_SPECULARENABLE, TRUE);
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      CreateImages                                                         */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void MidwayMathUI::CreateImages() {

LPDIRECT3DSURFACE8 backbuffer;
D3DDISPLAYMODE mode;
LPDIRECT3D8 d3d;
int w, h;

  mpRenderer->GetDevice()->GetDirect3D(&d3d);
  d3d->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &mode);

  mpRenderer->GetResolution(&w, &h);
  
  // create a new surface
  mpRenderer->GetDevice()->GetBackBuffer(0, D3DBACKBUFFER_TYPE_MONO, &backbuffer);
  mpRenderer->GetDevice()->CreateImageSurface(w, h, mode.Format, &mpBackgroundImage);

  HRESULT hr = D3DXLoadSurfaceFromFileA(mpBackgroundImage,
                                        NULL,
                                        NULL, 
                                        UI_TEXTURE_MAIN_MENU,
                                        NULL, 
                                        D3DX_DEFAULT, 
                                        0, 
                                        NULL);

  if(FAILED(hr)){

  #ifdef _DEBUG
    LOG(mpLog) << "Failed to load background image: " << UI_TEXTURE_MAIN_MENU << " Error: " << DXGetErrorDescription8(hr) << "\n\n";
  #endif
  
    mpBackgroundImage->Release();
    backbuffer->Release();
    d3d->Release();
  
    throw "Failed to load resource";
  }

  // Get viewport dimensions
  D3DVIEWPORT8 vp;
  mpRenderer->GetDevice()->GetViewport(&vp);
  float sx = (float)vp.Width;
  float sy = (float)vp.Height;
  
  mpRenderer->GetDevice()->CreateVertexBuffer(4*sizeof(SCREENVERTEX),
                                              D3DUSAGE_WRITEONLY, 
                                              (D3DFVF_XYZRHW|D3DFVF_DIFFUSE),
                                              D3DPOOL_MANAGED, 
                                              &mpViewport);
  
  // Setup dimensions for the viewport covering sqaure
  SCREENVERTEX *v;
  mpViewport->Lock( 0, 0, (BYTE**)&v, 0 );
  v[0].p = D3DXVECTOR4( 0,sy,0.0f,1.0f);
  v[1].p = D3DXVECTOR4( 0, 0,0.0f,1.0f);
  v[2].p = D3DXVECTOR4(sx,sy,0.0f,1.0f);
  v[3].p = D3DXVECTOR4(sx, 0,0.0f,1.0f);
  mpViewport->Unlock();

  backbuffer->Release();
  d3d->Release();
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      CreateFonts                                                          */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void MidwayMathUI::CreateFonts() {

  HRESULT hr;
  HDC hdc = CreateCompatibleDC(NULL);
  HFONT hFont;
  HFONT hFontOld;
  LOGFONT lf;

  ZeroMemory(&lf, sizeof(lf));
  lf.lfHeight = 48;
  lf.lfWidth = FW_BOLD;
  lf.lfStrikeOut = 1;
    
  lstrcpy(lf.lfFaceName, _T("Arial"));
  
  hFont = CreateFontIndirect(&lf);
  hFontOld = (HFONT)SelectObject(hdc, hFont); 

  hr = D3DXCreateText(mpRenderer->GetDevice(), 
                      hdc, 
                      _T("Midway Math"), 
                      0.001f, 
                      0.4f, 
                      &mpTitleText, 
                      NULL, 
                      NULL);

  if(FAILED(hr)){

  #ifdef _DEBUG
    LOG(mpLog) << "Failed to create title text" << " Error: " << DXGetErrorDescription8(hr) << "\n\n";
  #endif
  
    throw "Failed to create title text";
  }

  SelectObject(hdc, hFontOld);
  
  /* Set the blue color */
  D3DUtil_InitMaterial(mTitleMaterial, 0.0f, 0.0f, 1.0f );
  
  // Create the menu fonts
  mpFont = new CD3DFont( _T("Impact"), 48, 0L ); 
  mpFont->InitDeviceObjects(mpRenderer->GetDevice());
  mpFont->RestoreDeviceObjects();

  // Establish colors for selected vs. normal menu items
  D3DUtil_InitMaterial(mNormalMaterial,   1.0f, 1.0f, 1.0f, 0.5f);
  D3DUtil_InitMaterial(mSelectedMaterial, 1.0f, 1.0f, 0.0f, 1.0f);
  D3DUtil_InitMaterial(mGameFontMaterial, 1.0f, 1.0f, 0.0f, 0.7f);

  // Restore the textures
  mpRenderer->GetDevice()->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
  mpRenderer->GetDevice()->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
  mpRenderer->GetDevice()->SetTextureStageState(0, D3DTSS_COLOROP,   D3DTOP_MODULATE);
  mpRenderer->GetDevice()->SetTextureStageState(0, D3DTSS_MINFILTER, D3DTEXF_LINEAR);
  mpRenderer->GetDevice()->SetTextureStageState(0, D3DTSS_MAGFILTER, D3DTEXF_LINEAR);
  
  mpRenderer->GetDevice()->SetRenderState(D3DRS_ZENABLE, TRUE);
  mpRenderer->GetDevice()->SetRenderState(D3DRS_DITHERENABLE, TRUE);
  mpRenderer->GetDevice()->SetRenderState(D3DRS_SPECULARENABLE, TRUE);

  D3DLIGHT8 light;
  D3DUtil_InitLight(light, D3DLIGHT_DIRECTIONAL, 10.0f, -10.0f, 10.0f);
  mpRenderer->GetDevice()->SetLight(0, &light);
  mpRenderer->GetDevice()->LightEnable(0, TRUE);
}
