/*****************************************************************************/
/* File: ModelFileLoader.cpp                                                 */
/*                                                                           */
/* Description:                                                              */
/*                                                                           */
/* Loads an .x model file into a ModelNode object. Creates new model nodes   */
/* as needed if the .x model contains a mesh hierarchy.                      */
/*                                                                           */
/* It is important to note that this implementation of a model loader will   */
/* only operate correctly if the model is exported from 3D Studio MAX with a */
/* 'top-level' hierarchy _only_. This means that meshes more than one level  */
/* deep will not load correctly and may lead to unpredictable results. Care  */
/* should be take when exporting the model in .x format.                     */
/*                                                                           */
/* All portions Copyright (c) Austin Krauss, 2003.                           */
/*****************************************************************************/

                           /* MODIFICATION LOG */
/*****************************************************************************/
/*  Date    *  Rev # *  Who     *   Comment                                  */
/*---------------------------------------------------------------------------*/
/* 6/15/03  *   02   *  Austin  * Misc. Cleanup                              */
/*---------------------------------------------------------------------------*/
/* 6/11/03  *   01   *  Austin  * Major revision to load model hierarchy.    */
/*---------------------------------------------------------------------------*/
/* 4/27/03  *   00   *  Austin  * Created.                                   */
/*****************************************************************************/

/*****************************************************************************/
/*                                 HEADERS                                   */  
/*****************************************************************************/

#include "ModelFileLoader.h"
#include "rmxftmpl.h"
#include "rmxfguid.h"
#include "Dxerr8.h"

using namespace MidwayMath;

/*****************************************************************************/
/*                               CONSTRUCTORS                                */  
/*****************************************************************************/

#ifdef _DEBUG
ModelFileLoader::ModelFileLoader(Logger *pLog, Renderer *pRenderer, const char *pcFileName, ModelNode *pModelNode) {

  mpLog       = pLog;
  mpRenderer  = pRenderer;
  mpcFileName = pcFileName;
  mpNode      = pModelNode;
  mpCurrentNode = pModelNode;
}
#endif

ModelFileLoader::ModelFileLoader(Renderer *pRenderer, const char *pcFileName, ModelNode *pModelNode) {

  mpRenderer  = pRenderer;
  mpcFileName = pcFileName;
  mpNode      = pModelNode;
  mpCurrentNode = pModelNode;
}


/*****************************************************************************/
/*                               DESTRUCTOR                                  */  
/*****************************************************************************/

ModelFileLoader::~ModelFileLoader() {

}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      Load                                                                 */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*      n                                                                    */
/*          A pointer to the ModelNode which is populated with the geometry. */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*      Loads an .x file into the node n.                                    */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void ModelFileLoader::Load(SceneGraphNode *n) {

  mpNode = n;
  Load();
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      Load                                                                 */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*      file                                                                 */
/*          A pointer to a string .x file name which is to read from.        */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*      Loads an .x file from the given file name into the internal node     */
/*      pointer.                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void ModelFileLoader::Load(char *file) {

  mpcFileName = file;
  Load();
}

#if 0
void ModelFileLoader::LoadFlat() {

HRESULT hr;
    
// Direct3D model data
LPD3DXMESH mesh, mesh2;
LPD3DXBUFFER matBuf; 
LPD3DXBUFFER adjBuf;
LPDIRECT3DTEXTURE8 *textures;
D3DXMATERIAL *material;
DWORD numMats;

  // Load the model
  hr = D3DXLoadMeshFromX(mpcFileName,
                         D3DXMESH_MANAGED,
                         mpRenderer->GetDevice(),
                         &adjBuf,
                         &matBuf,
                         &numMats,
                         &mesh);

  if(FAILED(hr)) {

  #ifdef _DEBUG
    LOG(mpLog) << "Failed to load model file from: " << mpcFileName << " Error: " << DXGetErrorDescription8(hr) << "\n\n";
  #endif
    throw "Failed to load resource";
  }

/*
  // Optimize the mesh for performance
  hr = mesh->OptimizeInplace(D3DXMESHOPT_COMPACT | D3DXMESHOPT_ATTRSORT | D3DXMESHOPT_VERTEXCACHE,
                             (DWORD*)adjBuf->GetBufferPointer(), NULL, NULL, NULL);
  
  if(FAILED(hr)) {

  #ifdef _DEBUG
    LOG(mpLog) << "Call to OptimizeInplace failed. Error: " << DXGetErrorDescription8(hr) << "\n\n";
  #endif
    throw "Failed to load resource";
  }
*/
  
  // Load textures
  material = (D3DXMATERIAL *) matBuf->GetBufferPointer();
  textures = new LPDIRECT3DTEXTURE8[numMats];

  for(int i = 0; i < numMats; i++) {

    if(material[i].pTextureFilename) {
    
      hr = D3DXCreateTextureFromFile(mpRenderer->GetDevice(),
                                     material[i].pTextureFilename,
                                     &textures[i]); 

      if(FAILED(hr)) {

      #ifdef _DEBUG
        LOG(mpLog) << "Failed to load texture from: " << material[i].pTextureFilename << " Error: " << DXGetErrorDescription8(hr) << "\n\n";
      #endif
        throw "Failed to load resource";
      }
    }
    else
      textures[i] = NULL;
  }
  
  // Calculate vertex normals
  DWORD fvf = mesh->GetFVF();
  if (!(fvf & D3DFVF_NORMAL)) {

    fvf |= D3DFVF_NORMAL;
    mesh->CloneMeshFVF(0, fvf, mpRenderer->GetDevice(), &mesh2);
    
    hr = D3DXComputeNormals(mesh2, NULL);  
  
    if(FAILED(hr)) {

    #ifdef _DEBUG
      LOG(mpLog) << "Failed to compute normals from: " << mpcFileName << " Error: " << DXGetErrorDescription8(hr) << "\n\n";
    #endif
      throw "Failed to load resource";
    }
  
  mesh->Release();
  mesh = mesh2;
  }

  // And update our scene graph
  mpCurrentNode->SetMesh(mesh);     
  mpCurrentNode->SetMatBuf(matBuf);
  mpCurrentNode->SetAdjBuf(adjBuf);
  mpCurrentNode->SetTextures(textures);     
  mpCurrentNode->SetNumMaterials(numMats);  
}
#endif
/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      Load                                                                 */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*      Loads an .x file geometry and textures into the internal node        */
/*      pointer.                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void ModelFileLoader::Load() {

HRESULT hr;
LPDIRECTXFILE dxFile;
LPDIRECTXFILEENUMOBJECT dxEnum;
LPDIRECTXFILEDATA dxCurObj;

  //Load mesh heirarchy
  hr = DirectXFileCreate(&dxFile);

  if(FAILED(hr)) {

  #ifdef _DEBUG
    LOG(mpLog) << "Call to DirectXFileCreate() failed. Error: " << DXGetErrorDescription8(hr) << "\n\n";
  #endif
    throw "Call to DirectXFileCreate() failed";
  }

  // Register templates for d3drm.
  hr = dxFile->RegisterTemplates((LPVOID)D3DRM_XTEMPLATES,
                                 D3DRM_XTEMPLATE_BYTES);
  
  if(FAILED(hr)) {

  #ifdef _DEBUG
    LOG(mpLog) << "Call to RegisterTemplates() failed. Error: " << DXGetErrorDescription8(hr) << "\n\n";
  #endif
    throw "Call to RegisterTemplates() failed";
  }

  // Create enum object.
  hr = dxFile->CreateEnumObject((LPVOID)mpcFileName,
                                DXFILELOAD_FROMFILE,
                                &dxEnum);

  if(FAILED(hr)) {

  #ifdef _DEBUG
    LOG(mpLog) << "Call to CreateEnumObject() failed. Error: " << DXGetErrorDescription8(hr) << "\n\n";
  #endif
    throw "Call to CreateEnumObject() failed";
  }

  while(SUCCEEDED(dxEnum->GetNextDataObject(&dxCurObj))) {

    LoadModelData(dxCurObj);
    dxCurObj->Release();
  }

  dxEnum->Release();
  dxFile->Release();
}

/*
  while(SUCCEEDED(dxEnum->GetNextDataObject(&dxCurObj))) {

      const GUID* pGUID;


// Used for our scene graph
ModelNode *model = static_cast<ModelNode*>(mpNode);
ModelNode *child;
    
dxCurObj->GetType(&pGUID);
  if(*pGUID == TID_D3DRMFrameTransformMatrix) {

    D3DXMATRIX* pmatMatrix;
    DWORD cbSize;
    dxCurObj->GetData( NULL, &cbSize, (VOID**)&pmatMatrix );
  }
  
      if(*pGUID == TID_D3DRMMesh) {
        
         char name[256];
         unsigned long len = 256;
         dxCurObj->GetName(name, &len);
        
        if(!model->GetMesh())
          mpCurrentNode = model;
        
        else {
          
          
          child = new ModelNode(mpLog, mpRenderer, name, model);
          model->AttachChild(child);
          mpCurrentNode = child;
        }  
  
        LoadModelMesh(dxCurObj);
        dxCurObj->Release();
      }
  }
*/


void ModelFileLoader::LoadModelData(LPDIRECTXFILEDATA pData) {

const GUID* pGUID;
    
// Used for loading embedded objects 
LPDIRECTXFILEOBJECT        pSubObj;
//LPDIRECTXFILEDATAREFERENCE pDataRef;
LPDIRECTXFILEDATA   pChildData = NULL;
//LPDIRECTXFILEDATA            pSubData = NULL;

// Used for our scene graph
ModelNode *model = static_cast<ModelNode*>(mpNode);
ModelNode *child;
char name[256];
unsigned long len = 256;

  pData->GetType(&pGUID);

  if(*pGUID == TID_D3DRMFrame) {

    pData->GetName(name, &len);
    if(len == 0)
      goto load;
    if(!model->GetMesh())
      mpCurrentNode = model;
    
    else {
//      pData->GetName(name, &len);
      child = new ModelNode(mpLog, mpRenderer, name, model);
      model->AttachChild(child);
      mpCurrentNode = child;
    }  
  #if 0 //_DEBUG
    LOG(mpLog) << "TID_D3DRMFrame found\n\n";
  #endif
  }

  else if(*pGUID == TID_D3DRMFrameTransformMatrix) {

    D3DXMATRIX* pmatMatrix;
    DWORD cbSize;
    pData->GetData( NULL, &cbSize, (VOID**)&pmatMatrix );

    // Assume uniform scaling
    //if(mpCurrentNode == model)  
    //  mpCurrentNode->mfLocalScale = pmatMatrix->_11;

    mpCurrentNode->mLocalTranslation.x = pmatMatrix->_41;
    mpCurrentNode->mLocalTranslation.y = pmatMatrix->_42;
    mpCurrentNode->mLocalTranslation.z = pmatMatrix->_43;
  }
  
  else if(*pGUID == TID_D3DRMMesh) {

    LoadModelMesh(pData);
  }

  else {
  #if 0 //_DEBUG
    LOG(mpLog) << "GUID found: " << pGUID << "\n\n";
  #endif
  }
load: 
  // Scan for embedded templates
  while(SUCCEEDED(pData->GetNextObject(&pSubObj))) {

    // Process embedded references
    if(SUCCEEDED(pSubObj->QueryInterface(IID_IDirectXFileData, (void**)&pChildData))) {
    
      LoadModelData(pChildData);
      pChildData->Release();
    }
  }
}

void ModelFileLoader::LoadModelMesh(LPDIRECTXFILEDATA pMesh) {

HRESULT hr;
    
// Direct3D model data
LPD3DXMESH mesh, mesh2;
LPD3DXBUFFER matBuf; 
LPD3DXBUFFER adjBuf;
LPDIRECT3DTEXTURE8 *textures;
D3DXMATERIAL *material;
DWORD numMats;

  // Load the model
  hr = D3DXLoadMeshFromXof(pMesh,
                           D3DXMESH_MANAGED,
                           mpRenderer->GetDevice(),
                           &adjBuf,
                           &matBuf,
                           &numMats,
                           &mesh);

  if(FAILED(hr)) {

  #ifdef _DEBUG
    LOG(mpLog) << "Failed to load model file from: " << mpcFileName << " Error: " << DXGetErrorDescription8(hr) << "\n\n";
  #endif
    throw "Failed to load resource";
  }

/*
  // Optimize the mesh for performance
  hr = mesh->OptimizeInplace(D3DXMESHOPT_COMPACT | D3DXMESHOPT_ATTRSORT | D3DXMESHOPT_VERTEXCACHE,
                             (DWORD*)adjBuf->GetBufferPointer(), NULL, NULL, NULL);
  
  if(FAILED(hr)) {

  #ifdef _DEBUG
    LOG(mpLog) << "Call to OptimizeInplace failed. Error: " << DXGetErrorDescription8(hr) << "\n\n";
  #endif
    throw "Failed to load resource";
  }
*/
  
  // Load textures
  material = (D3DXMATERIAL *) matBuf->GetBufferPointer();
  textures = new LPDIRECT3DTEXTURE8[numMats];

  for(int i = 0; i < numMats; i++) {

    if(material[i].pTextureFilename) {
    
      std::string path = ".\\Textures\\";
      path += material[i].pTextureFilename;
      hr = D3DXCreateTextureFromFile(mpRenderer->GetDevice(),
                                     path.c_str(),/*material[i].pTextureFilename,*/
                                     &textures[i]); 

      if(FAILED(hr)) {

      #ifdef _DEBUG
        LOG(mpLog) << "Failed to load texture from: " << material[i].pTextureFilename << " Error: " << DXGetErrorDescription8(hr) << "\n\n";
      #endif
        throw "Failed to load resource";
      }
    }
    else
      textures[i] = NULL;
  }
  
  // Calculate vertex normals
  DWORD fvf = mesh->GetFVF();
  if (!(fvf & D3DFVF_NORMAL)) {

    fvf |= D3DFVF_NORMAL;
    mesh->CloneMeshFVF(0, fvf, mpRenderer->GetDevice(), &mesh2);
    
    hr = D3DXComputeNormals(mesh2, NULL);  
  
    if(FAILED(hr)) {

    #ifdef _DEBUG
      LOG(mpLog) << "Failed to compute normals from: " << mpcFileName << " Error: " << DXGetErrorDescription8(hr) << "\n\n";
    #endif
      throw "Failed to load resource";
    }
  
  mesh->Release();
  mesh = mesh2;
  }

#if 0 // Compute initial position

LPDIRECT3DVERTEXBUFFER8 pVB;
DWORD        dwNumVertices;
BYTE*        pVertices;
FLOAT        fRadius;

  // Gain access to the model's vertices
  mesh->GetVertexBuffer( &pVB );
  dwNumVertices = mesh->GetNumVertices();
  hr = pVB->Lock(0, 0, &pVertices, 0);

  if(FAILED(hr)) {

  #ifdef _DEBUG
    LOG(mpLog) << "Failed to compute normals from: " << mpcFileName << " Error: " << DXGetErrorDescription8(hr) << "\n\n";
  #endif
    throw "Failed to load resource";
  }

  hr = D3DXComputeBoundingSphere(pVertices,
                                 dwNumVertices,
                                 fvf,
                                 &mpCurrentNode->mPosition,
                                 &fRadius);
                                    
  if(FAILED(hr)) {

  #ifdef _DEBUG
    LOG(mpLog) << "Failed to compute normals from: " << mpcFileName << " Error: " << DXGetErrorDescription8(hr) << "\n\n";
  #endif
    throw "Failed to load resource";
  }

#endif

  
  
  // And update our scene graph
  mpCurrentNode->SetMesh(mesh);     
  mpCurrentNode->SetMatBuf(matBuf);
  mpCurrentNode->SetAdjBuf(adjBuf);
  mpCurrentNode->SetTextures(textures);     
  mpCurrentNode->SetNumMaterials(numMats);  
}
