/*****************************************************************************/
/* File: Renderer.cpp                                                        */
/*                                                                           */
/* Description:                                                              */
/*                                                                           */
/* Object responsible for the main rendering subsystem. Creates Direct3D     */
/* rendering objects from given parameters of resolution and fullscreen mode.*/
/* Contains access and utility functions for objects to use for rendering    */
/* their geometry. Contains a reference to the root Scene Graph node for     */
/* rendering geometry.                                                       */
/*                                                                           */
/* All portions Copyright (c) Austin Krauss, 2003.                           */
/*****************************************************************************/

                           /* MODIFICATION LOG */
/*****************************************************************************/
/*  Date    *  Rev # *  Who     *   Comment                                  */
/*---------------------------------------------------------------------------*/
/* 6/15/03  *   01   *  Austin  * Misc. cleanup                              */
/*---------------------------------------------------------------------------*/
/* 4/20/03  *   00   *  Austin  * Created.                                   */
/*****************************************************************************/

/*****************************************************************************/
/*                                 HEADERS                                   */  
/*****************************************************************************/

#include "Renderer.h"

using namespace MidwayMath;

/*****************************************************************************/
/*                                CONSTANTS                                  */  
/*****************************************************************************/

const int Renderer::RENDERER_RES_640X480   = 1;  
const int Renderer::RENDERER_RES_800X600   = 2;  
const int Renderer::RENDERER_RES_1024X768  = 3; 
const int Renderer::RENDERER_RES_1280X1024 = 4;

/*****************************************************************************/
/*                               CONSTRUCTORS                                */  
/*****************************************************************************/

#ifdef _DEBUG
Renderer::Renderer(Logger *pLog, HWND hWnd, int res, bool fs) 
    :mpLog(pLog),
     mhWnd(hWnd),
     mbFullScreen(fs),
     mbShowStatistics(false),
     mpD3D8Object(NULL),
     mpD3D8Device(NULL) {

  Initialize(res);
}
#endif

Renderer::Renderer(HWND hWnd, int res, bool fs)
    :mhWnd(hWnd),
     mbFullScreen(fs), 
     mbShowStatistics(false),
     mpD3D8Object(NULL),
     mpD3D8Device(NULL) {

  Initialize(res);
}

/*****************************************************************************/
/*                               DESTRUCTOR                                  */  
/*****************************************************************************/

Renderer::~Renderer() {

  mpStatsFont->InvalidateDeviceObjects();
  mpStatsFont->DeleteDeviceObjects();
  mpD3D8Device->Release();
  mpD3D8Object->Release();
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      Initialize                                                           */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*      resolution                                                           */
/*          One of the predefined resolution constants which to be the       */
/*          screen resolution.                                               */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*      Creates the Direct3D device from the passed resolution.              */
/*      class. A display mode list is built from the default video card's    */
/*      supported hardware acceleration features. A stencil/depth format of  */
/*      D3DFMT_D16 is hardcoded here and must be changed if another buffer   */
/*      format is needed. Note that this is 16-bit color.                    */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void Renderer::Initialize(int resolution) {

HRESULT hr;
int desiredHeight, desiredWidth;
std::vector <D3DDISPLAYMODE>::iterator currDisplayMode;
D3DDISPLAYMODE displayMode;

  switch(resolution) {

    case RENDERER_RES_640X480:
      desiredWidth = 640; desiredHeight = 480;
    break;    
    case RENDERER_RES_800X600:  
      desiredWidth = 800; desiredHeight = 600;
    break;    
    case RENDERER_RES_1024X768: 
      desiredWidth = 1024; desiredHeight = 768;
    break;    
    case RENDERER_RES_1280X1024:
      desiredWidth = 1280; desiredHeight = 1024;
    break;    
  
    default:
      #ifdef _DEBUG
        LOG(mpLog) << "Requested resolution undefined: " << resolution << "\n\n";
      #endif
      throw "Requested resolution undefined";
    break;
  }
  
  miResolution = resolution;
  
  //Do the Direct 3D initialization
  mpD3D8Object = Direct3DCreate8(D3D_SDK_VERSION);

  if(!mpD3D8Object) {

  #ifdef _DEBUG
    LOG(mpLog) << "Call to Direc3DCreate8 returned <null>.\n\n";
  #endif
    throw "Call to Direc3DCreate8 failed";
  }
  
  /* Get a list of display modes supported by this adapter (filtering out 
   * resolutions we aren't interested in)
   */
  BuildDisplayModeList(D3DADAPTER_DEFAULT);
  
  if(mDisplayModeList.size() <= 0) {
  
  #ifdef _DEBUG
    LOG(mpLog) << "Insufficient number of display modes: " << mDisplayModeList.size() << "\n\n";
  #endif
    throw "Insufficient number of display modes";
  }
  
  // Remove the modes from the list that don't support our device format
  for(currDisplayMode = mDisplayModeList.begin(); currDisplayMode != mDisplayModeList.end(); currDisplayMode++) {

    if(currDisplayMode->Format != D3DFMT_R5G6B5) { // 16-bit color
      
      mDisplayModeList.erase(currDisplayMode--);
      continue;
    }
    
    hr = mpD3D8Object->CheckDeviceType(D3DADAPTER_DEFAULT,
                                       D3DDEVTYPE_HAL,
                                       currDisplayMode->Format,
                                       currDisplayMode->Format,
                                       !mbFullScreen);

    if(hr == D3D_OK) {

      hr = mpD3D8Object->CheckDeviceFormat(D3DADAPTER_DEFAULT,
                                       D3DDEVTYPE_HAL,
                                       currDisplayMode->Format,
                                       D3DUSAGE_DEPTHSTENCIL,
                                       D3DRTYPE_SURFACE,
                                       D3DFMT_D16);
      if(hr == D3D_OK) {

        hr = mpD3D8Object->CheckDepthStencilMatch(D3DADAPTER_DEFAULT,
                                              D3DDEVTYPE_HAL,
                                              currDisplayMode->Format,
                                              currDisplayMode->Format,
                                              D3DFMT_D16);

        if(hr == D3D_OK)
          continue;
      }
    }   
  
    mDisplayModeList.erase(currDisplayMode--);
  }
  
#ifdef _DEBUG
  LOG(mpLog) << "Display mode list contains " << mDisplayModeList.size() << " display modes:\n\n";
  
  int num = 1;
  for(currDisplayMode = mDisplayModeList.begin(); currDisplayMode != mDisplayModeList.end(); currDisplayMode++, num++) {

#if 1
    LOG(mpLog) << "Display Mode #" << num << ":\n"                          
               << "\tWidth: " << currDisplayMode->Width << "\n"                         
               << "\tHeight: " << currDisplayMode->Height << "\n"                   
               << "\tRefresh Rate (Hz): " << currDisplayMode->RefreshRate << "\n"   
               << "\tFormat: " << currDisplayMode->Format << "\n\n";
#endif
  }
#endif

  // Check to see if we've filtered all the candidates
  if(mDisplayModeList.size() <= 0) {
  
  #ifdef _DEBUG
    LOG(mpLog) << "Insufficient number of display modes: " << mDisplayModeList.size() << "\n\n";
  #endif
    throw "Insufficient number of display modes";
  }
  
  // Finally pick the display mode with the resolution requested
  for(currDisplayMode = mDisplayModeList.begin(); currDisplayMode != mDisplayModeList.end(); currDisplayMode++) {

    if(currDisplayMode->Width == desiredWidth && currDisplayMode->Height == desiredHeight)
      break;
  }
  
#ifdef _DEBUG
  if(mbFullScreen) {
    LOG(mpLog) << "Using Display Mode:\n"
               << "\tWidth: " << currDisplayMode->Width << "\n"                         
               << "\tHeight: " << currDisplayMode->Height << "\n"                   
               << "\tRefresh Rate (Hz): " << currDisplayMode->RefreshRate << "\n"   
               << "\tFormat: " << currDisplayMode->Format << "\n\n";
  }
#endif

  // OK create the device
  D3DPRESENT_PARAMETERS pp;

  if(mbFullScreen) {
    pp.BackBufferWidth  = currDisplayMode->Width;
    pp.BackBufferHeight = currDisplayMode->Height;
    pp.BackBufferFormat = currDisplayMode->Format;
  }
  else { // Special Considerations if not full screen
    pp.BackBufferWidth  = 640;
    pp.BackBufferHeight = 480;
    mpD3D8Object->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &displayMode);
    pp.BackBufferFormat = displayMode.Format;
  }
  
  pp.BackBufferCount                 = 1;
  pp.MultiSampleType                 = D3DMULTISAMPLE_NONE;
  pp.SwapEffect                      = D3DSWAPEFFECT_DISCARD;
  pp.hDeviceWindow                   = mhWnd;
  pp.Windowed                        = !mbFullScreen;
  pp.EnableAutoDepthStencil          = true;
  pp.AutoDepthStencilFormat          = D3DFMT_D16; // No stencil buffer with this mode
  pp.Flags                           = 0;
  pp.FullScreen_RefreshRateInHz      = 0;
  pp.FullScreen_PresentationInterval = 0;

  hr = mpD3D8Object->CreateDevice(D3DADAPTER_DEFAULT,
                                  D3DDEVTYPE_HAL,
                                  mhWnd,
                                  D3DCREATE_HARDWARE_VERTEXPROCESSING,
                                  &pp,
                                  &mpD3D8Device);

  if(!mpD3D8Device) {

  #ifdef _DEBUG
    LOG(mpLog) << "Call to CreateDevice returned <null>.\n\n";
  #endif
    throw "Unable to create Direct3D Device. Check hardware acceleration requirements";
  }

  // Create the Font that renders the statistics
  mpStatsFont = new CD3DFont(_T("Arial"), 10, D3DFONT_BOLD/*0*/);
  mpStatsFont->InitDeviceObjects(mpD3D8Device);
  mpStatsFont->RestoreDeviceObjects();

  // Fill in some device and mode information
  D3DADAPTER_IDENTIFIER8 deviceInfo;
  mpD3D8Object->GetAdapterIdentifier(D3DADAPTER_DEFAULT, D3DENUM_NO_WHQL_LEVEL, &deviceInfo);
  
  sprintf(mcDeviceInfo, "Device: %s\nDriver: %s Version: %d.%d.%d.%d", 
    deviceInfo.Description, deviceInfo.Driver, 
    HIWORD(deviceInfo.DriverVersion.HighPart), 
    LOWORD(deviceInfo.DriverVersion.HighPart),
    HIWORD(deviceInfo.DriverVersion.LowPart),
    LOWORD(deviceInfo.DriverVersion.LowPart));
  
  mpD3D8Device->GetDisplayMode(&displayMode);
  sprintf(mcModeInfo, "Format: %d x %d (%dHz) ", displayMode.Width, displayMode.Height, displayMode.RefreshRate);

  switch(displayMode.Format) {

    case D3DFMT_R8G8B8:      strcat(mcModeInfo, "D3DFMT_R8G8B8");      break;    
    case D3DFMT_A8R8G8B8:    strcat(mcModeInfo, "D3DFMT_A8R8G8B8");    break;
    case D3DFMT_X8R8G8B8:    strcat(mcModeInfo, "D3DFMT_X8R8G8B8");    break;
    case D3DFMT_R5G6B5:      strcat(mcModeInfo, "D3DFMT_R5G6B5");      break;
    case D3DFMT_X1R5G5B5:    strcat(mcModeInfo, "D3DFMT_X1R5G5B5");    break;
    case D3DFMT_A1R5G5B5:    strcat(mcModeInfo, "D3DFMT_A1R5G5B5");    break;
    case D3DFMT_A4R4G4B4:    strcat(mcModeInfo, "D3DFMT_A4R4G4B4");    break;
    case D3DFMT_R3G3B2:      strcat(mcModeInfo, "D3DFMT_R3G3B2");      break;
    case D3DFMT_A8:          strcat(mcModeInfo, "D3DFMT_A8");          break;
    case D3DFMT_A8R3G3B2:    strcat(mcModeInfo, "D3DFMT_A8R3G3B2");    break;
    case D3DFMT_X4R4G4B4:    strcat(mcModeInfo, "D3DFMT_X4R4G4B4");    break;
    case D3DFMT_A2B10G10R10: strcat(mcModeInfo, "D3DFMT_A2B10G10R10"); break;
    case D3DFMT_G16R16:      strcat(mcModeInfo, "D3DFMT_G16R16");      break;
    default:                 strcat(mcModeInfo, "D3DFMT_UNKNOWN");     break;
  }

  D3DXCreateTextureFromFileA(mpD3D8Device,".//Textures//Right.bmp",&mpSkybox[0]);
  D3DXCreateTextureFromFileA(mpD3D8Device,".//Textures//Left.bmp",&mpSkybox[1]);
  D3DXCreateTextureFromFileA(mpD3D8Device,".//Textures//Top.bmp",&mpSkybox[2]);
  D3DXCreateTextureFromFileA(mpD3D8Device,".//Textures//Bottom.bmp",&mpSkybox[3]);
  D3DXCreateTextureFromFileA(mpD3D8Device,".//Textures//Back.bmp",&mpSkybox[4]);
  D3DXCreateTextureFromFileA(mpD3D8Device,".//Textures//Front.bmp",&mpSkybox[5]);

//  D3DXCreateTextureFromFileA(mpD3D8Device,".//Textures//Front.jpg",&mpSkybox[0]);
//  D3DXCreateTextureFromFileA(mpD3D8Device,".//Textures//Back.jpg",&mpSkybox[1]);
//  D3DXCreateTextureFromFileA(mpD3D8Device,".//Textures//Top.jpg",&mpSkybox[2]);
//  D3DXCreateTextureFromFileA(mpD3D8Device,".//Textures//Bottom.jpg",&mpSkybox[3]);
//  D3DXCreateTextureFromFileA(mpD3D8Device,".//Textures//Right.jpg",&mpSkybox[4]);
//  D3DXCreateTextureFromFileA(mpD3D8Device,".//Textures//Left.jpg",&mpSkybox[5]);
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      BuildDisplayModeList                                                 */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*      resolution                                                           */
/*          One of the predefined resolution constants which to be the       */
/*          screen resolution.                                               */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*      Creates a <vector> of display mode lists from the adapter that is    */
/*      passed. Filters out resolutions that we aren't interested in.        */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void Renderer::BuildDisplayModeList(UINT adapter) {

int numModes;

  numModes = mpD3D8Object->GetAdapterModeCount(adapter);

  if(numModes == 0) {

  #ifdef _DEBUG
    LOG(mpLog) << "GetAdapterModeCount returned 0, check adapter type\n\n";
  #endif

    throw "GetAdapterModeCount returned 0, check adapter type";
  }
   
  #ifdef _DEBUG
    LOG(mpLog) << "D3D Adapter #" << adapter << " has " << numModes << " display modes\n\n";
  #endif

  for(int i = 0; i < numModes; i++) {

    D3DDISPLAYMODE dmode;
    mpD3D8Object->EnumAdapterModes(adapter, i, &dmode);
    
    if(dmode.Width == 640 && dmode.Height == 480) 
      mDisplayModeList.push_back(dmode);
    else if (dmode.Width == 800 && dmode.Height == 600)
      mDisplayModeList.push_back(dmode);
    else if (dmode.Width == 1024 && dmode.Height == 768)
      mDisplayModeList.push_back(dmode);
    else if (dmode.Width == 1280 && dmode.Height == 1024)
      mDisplayModeList.push_back(dmode);
  }
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      GetResolution                                                        */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void Renderer::GetResolution(int *width, int *height) {

  switch(miResolution) {

    case RENDERER_RES_640X480:
      *width = 640; *height = 480;
    break;    
    case RENDERER_RES_800X600:  
      *width = 800; *height = 600;
    break;    
    case RENDERER_RES_1024X768: 
      *width = 1024; *height = 768;
    break;    
    case RENDERER_RES_1280X1024:
      *width = 1280; *height = 1024;
    break;    
  
    default:
      #ifdef _DEBUG
        LOG(mpLog) << "Resolution undefined: " << miResolution << "\n\n";
      #endif
      throw "Resolution undefined";
    break;
  }
}




/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      Render                                                               */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*      Begins the rendering process. The Renderer renders any internal items*/
/*      before and/or after calling into the Scene Graph.                    */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void Renderer::Render() {

#ifdef _DEBUG
  if(mbShowStatistics)
    CalculateStatistics();
#endif
  
    mpD3D8Device->Clear(0L, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB(0,0,255), 1.0f, 0L);
    
    // Draw the skybox
    DrawSkybox(&mpCamera->GetViewMatrix(), 10.0f);

  #ifdef _DEBUG
    // Draw world axis
  if(mbShowStatistics)
    DrawAxis(); 
  #endif    
    
  //Draw the root node
  mpRootNode->Draw();

  
  
  // Draw the mouse cursor
#if 0
  D3DXVECTOR3 pt = mpCamera->GetLookatPt();
D3DXMATRIX matd3d;

  mpD3D8Device->SetRenderState(D3DRS_LIGHTING, FALSE);
  D3DXMatrixIdentity(&matd3d);

  mpD3D8Device->SetTransform(D3DTS_WORLD, &matd3d);
  mpD3D8Device->SetVertexShader(D3DFVF_XYZ | D3DFVF_DIFFUSE);
  mpD3D8Device->SetTexture(0,NULL);

  struct LitPointVertex
  {
    float x, y, z;
    DWORD color;
  };
  
  LitPointVertex x[2];
  LitPointVertex y[2];
  x[0].color = D3DCOLOR_XRGB(255,0,0);
  x[1].color = D3DCOLOR_XRGB(255,0,0);
  y[0].color = D3DCOLOR_XRGB(0,255,0);
  y[1].color = D3DCOLOR_XRGB(0,255,0);

  x[0].x = pt.x-2; x[0].y = pt.y; x[0].z = pt.z-100;
  y[0].x = pt.x; y[0].y = pt.y+2; y[0].z = pt.z-100;    
  
  // x axis
  x[1].x = pt.x+2;  x[1].y = pt.y; x[1].z = pt.z-100;
  // y axis
  y[1].x = pt.x; y[1].y = pt.y-2; y[1].z = pt.z-100;

  mpD3D8Device->DrawPrimitiveUP(D3DPT_LINELIST, 1, x, sizeof(LitPointVertex));
  mpD3D8Device->DrawPrimitiveUP(D3DPT_LINELIST, 1, y, sizeof(LitPointVertex));
  mpD3D8Device->SetRenderState(D3DRS_LIGHTING, TRUE);

  ///////////////////////
#endif  
  
   #ifdef _DEBUG    
    if(mbShowStatistics)
      mpStatsFont->DrawText(0, 0, D3DCOLOR_XRGB(255,0,0), mcStats);
  #endif
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      BeginRenderMenu                                                      */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void Renderer::BeginRenderMenu() {

#ifdef _DEBUG
  if(mbShowStatistics)
    CalculateStatistics();
#endif
  
  // Render the actual scene
  mpD3D8Device->BeginScene();

    mpD3D8Device->Clear(0L, NULL, D3DCLEAR_TARGET|D3DCLEAR_ZBUFFER, D3DCOLOR_XRGB(0,0,255), 1.0f, 0L);
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      EndRenderMenu                                                        */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void Renderer::EndRenderMenu() {

  #ifdef _DEBUG 
    if(mbShowStatistics)
      mpStatsFont->DrawText(0, 0, D3DCOLOR_XRGB(255,0,0), mcStats);
  #endif
  
  mpD3D8Device->EndScene();
  
  // Show the frame on the primary surface.
  mpD3D8Device->Present(NULL, NULL, NULL, NULL);
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      CalculateStatistics                                                  */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*      Creates a statistics string that will be displayed on the screen by  */
/*      the Render() function if the user wishes stats to be displayed. This */
/*      function is responsible for calculating the framerate.               */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void Renderer::CalculateStatistics() {

static float lastTime = 0.0f;
static DWORD frames = 0L;
char buf[256];
  
  frames++; //Assume the rendering succeeds as normal

  // Update the scene stats once per second
  float currTime = DXUtil_Timer(TIMER_GETABSOLUTETIME);
  if((currTime - lastTime) > 1.0f) {

    float fps = frames / (currTime - lastTime);
    lastTime = currTime;
    frames = 0L;
    sprintf(mcStats, "FPS: %.02f %s\n%s", fps, mcModeInfo, mcDeviceInfo);

    // Camera stats
    D3DXVECTOR3 pos = mpCamera->GetEyePt();
    D3DXVECTOR3 eye = mpCamera->GetLookatPt();

    sprintf(buf, "\n\nCamera Position: (%.2f,%.2f,%.2f)", pos.x, pos.y, pos.z);
    strcat(mcStats, buf);
    sprintf(buf, "\nCamera Look At Point: (%.2f,%.2f,%.2f)", eye.x, eye.y, eye.z);
    strcat(mcStats, buf);
  
    // Scene Graph Stats
    sprintf(buf, "\n\nNodes in Scene Graph: %d", mpRootNode->GetNumberOfSubtreeNodes() + 1); // +1 for root node
    strcat(mcStats, buf);
  }
}

/*****************************************************************************/
/*  FUNCTION:                                                                */
/*                                                                           */
/*      DrawAxis                                                             */
/*                                                                           */
/*  PARAMETERS:                                                              */
/*                                                                           */
/*  DESCRIPTION:                                                             */
/*                                                                           */
/*      Draws the world X, Y, and Z axis in red, green, and yellow colors    */
/*      respectively. Used for reference.                                    */
/*                                                                           */
/*  RETURNS:                                                                 */
/*                                                                           */
/*****************************************************************************/

void Renderer::DrawAxis() {

D3DXMATRIX matd3d;

  D3DXMatrixIdentity(&matd3d);

  mpD3D8Device->SetTransform(D3DTS_WORLD, &matd3d);
  mpD3D8Device->SetVertexShader(D3DFVF_XYZ | D3DFVF_DIFFUSE);
  mpD3D8Device->SetTexture(0,NULL);
  mpD3D8Device->SetRenderState(D3DRS_LIGHTING, FALSE);

  struct LitPointVertex
  {
    float x, y, z;
    DWORD color;
  };
  
  LitPointVertex x[2];
  LitPointVertex y[2];
  LitPointVertex z[2];
  x[0].color = D3DCOLOR_XRGB(255,0,0);
  x[1].color = D3DCOLOR_XRGB(255,0,0);
  y[0].color = D3DCOLOR_XRGB(0,255,0);
  y[1].color = D3DCOLOR_XRGB(0,255,0);
  z[0].color = D3DCOLOR_XRGB(255,255,0);
  z[1].color = D3DCOLOR_XRGB(255,255,0);

  x[0].x = x[0].y = x[0].z = 0; 
  y[0].x = y[0].y = y[0].z = 0; 
  z[0].x = z[0].y = z[0].z = 0;
  
  // x axis
  x[1].x = 1500;    x[1].y = 0; x[1].z = 0;
  // y axis
  y[1].x = 0;   y[1].y = 1500; y[1].z = 0;
  // z axis
  z[1].x = 0;   z[1].y = 0; z[1].z = 1500;

  mpD3D8Device->DrawPrimitiveUP(D3DPT_LINELIST, 1, x, sizeof(LitPointVertex));
  mpD3D8Device->DrawPrimitiveUP(D3DPT_LINELIST, 1, y, sizeof(LitPointVertex));
  mpD3D8Device->DrawPrimitiveUP(D3DPT_LINELIST, 1, z, sizeof(LitPointVertex));
  mpD3D8Device->SetRenderState(D3DRS_LIGHTING, TRUE);
}

bool Renderer::DrawSkybox(D3DXMATRIX *view, float Size) {

    //center view matrix for skybox
    D3DXMATRIX adj_view = *view; 
    adj_view._41 = 0.0f; adj_view._42 = 0.0f; adj_view._43 = 0.0f;
    mpD3D8Device->SetTransform(D3DTS_VIEW, &adj_view);

    //set world matrix to identity
    D3DXMATRIX ident; D3DXMatrixIdentity(&ident);
    mpD3D8Device->SetTransform(D3DTS_WORLD, &ident);

    //turn off lighting
    mpD3D8Device->SetRenderState(D3DRS_LIGHTING, FALSE);

    //turn off depth buffer writes
    mpD3D8Device->SetRenderState(D3DRS_ZWRITEENABLE, FALSE);

    //set the vertex streams
    mpD3D8Device->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
    mpD3D8Device->SetVertexShader(D3DFVF_XYZ | D3DFVF_DIFFUSE | D3DFVF_TEX1);

    //clamp textures
    mpD3D8Device->SetTextureStageState(0, D3DTSS_ADDRESSU, D3DTADDRESS_CLAMP);
    mpD3D8Device->SetTextureStageState(0, D3DTSS_ADDRESSV, D3DTADDRESS_CLAMP);

    //use the texture for color info
    mpD3D8Device->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
    mpD3D8Device->SetTextureStageState(0, D3DTSS_COLOROP,   D3DTOP_SELECTARG1);

    //set the texture maps and draw the quads appropriately
    struct LitVertex
    {
        float x,y,z;
        DWORD color;
        float u,v;
    };
    LitVertex Mesh[4];
    DWORD Color = 0xFFFFFFFF;

    //front
    Mesh[0].x = -Size; Mesh[0].y = Size;  Mesh[0].z = Size;
    Mesh[0].u = 0.0f;  Mesh[0].v = 0.0f;
    Mesh[0].color = Color;

    Mesh[1].x = Size;  Mesh[1].y = Size;    Mesh[1].z = Size;
    Mesh[1].u = 1.0f;  Mesh[1].v = 0.0f;
    Mesh[1].color = Color;

    Mesh[2].x = -Size;  Mesh[2].y = -Size; Mesh[2].z = Size;
    Mesh[2].u = 0.0f;   Mesh[2].v = 1.0f;
    Mesh[2].color = Color;

    Mesh[3].x = Size; Mesh[3].y = -Size; Mesh[3].z = Size;
    Mesh[3].u = 1.0f; Mesh[3].v = 1.0f;
    Mesh[3].color = Color;

    mpD3D8Device->SetTexture(0, mpSkybox[0]);
    mpD3D8Device->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, Mesh, sizeof(LitVertex));

    //back
    Mesh[0].x = Size; Mesh[0].y = Size;  Mesh[0].z = -Size;
    Mesh[0].u = 0.0f;     Mesh[0].v = 0.0f;
    Mesh[0].color = Color;

    Mesh[1].x = -Size;  Mesh[1].y = Size;   Mesh[1].z = -Size;
    Mesh[1].u = 1.0f;     Mesh[1].v = 0.0f;
    Mesh[1].color = Color;

    Mesh[2].x = Size;  Mesh[2].y = -Size; Mesh[2].z = -Size;
    Mesh[2].u = 0.0f;     Mesh[2].v = 1.0f;
    Mesh[2].color = Color;

    Mesh[3].x = -Size; Mesh[3].y = -Size; Mesh[3].z = -Size;
    Mesh[3].u = 1.0f;     Mesh[3].v = 1.0f;
    Mesh[3].color = Color;

    mpD3D8Device->SetTexture(0, mpSkybox[1]);
    mpD3D8Device->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, Mesh, sizeof(LitVertex));

    //top
    Mesh[0].x = Size; Mesh[0].y = Size;  Mesh[0].z = -Size;
    Mesh[0].u = 0.0f;     Mesh[0].v = 0.0f;
    Mesh[0].color = Color;

    Mesh[1].x = -Size;  Mesh[1].y = Size;   Mesh[1].z = -Size;
    Mesh[1].u = 0.0f;     Mesh[1].v = 1.0f;
    Mesh[1].color = Color;

    Mesh[2].x = Size; Mesh[2].y = Size; Mesh[2].z = Size;
    Mesh[2].u = 1.0f;     Mesh[2].v = 0.0f;
    Mesh[2].color = Color;

    Mesh[3].x = -Size;  Mesh[3].y = Size; Mesh[3].z = Size;
    Mesh[3].u = 1.0f;     Mesh[3].v = 1.0f;
    Mesh[3].color = Color;

    mpD3D8Device->SetTexture(0, mpSkybox[2]);
    mpD3D8Device->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, Mesh, sizeof(LitVertex));

    //bottom
    Mesh[0].x = -Size;  Mesh[0].y  = -Size; Mesh[0].z = -Size;
    Mesh[0].u = 0.0f;     Mesh[0].v = 0.0f;
    Mesh[0].color = Color;

    Mesh[1].x = -Size;  Mesh[1].y = -Size; Mesh[1].z = Size;
    Mesh[1].u = 1.0f;      Mesh[1].v = 0.0f;
    Mesh[1].color = Color;

    Mesh[2].x = Size;  Mesh[2].y  = -Size; Mesh[2].z = -Size;
    Mesh[2].u = 0.0f;     Mesh[2].v = 1.0f;
    Mesh[2].color = Color;

    Mesh[3].x = Size;  Mesh[3].y = -Size; Mesh[3].z = Size;
    Mesh[3].u = 1.0f;      Mesh[3].v = 1.0f;
    Mesh[3].color = Color;

    mpD3D8Device->SetTexture(0, mpSkybox[3]);
    mpD3D8Device->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, Mesh, sizeof(LitVertex));

    //right
    Mesh[0].x = Size; Mesh[0].y = Size;   Mesh[0].z = Size;
    Mesh[0].u = 0.0f; Mesh[0].v = 0.0f;
    Mesh[0].color = Color;

    Mesh[1].x = Size;  Mesh[1].y = Size;  Mesh[1].z = -Size;
    Mesh[1].u = 1.0f;  Mesh[1].v = 0.0f;
    Mesh[1].color = Color;

    Mesh[2].x = Size;  Mesh[2].y = -Size; Mesh[2].z = Size;
    Mesh[2].u = 0.0f;  Mesh[2].v = 1.0f;
    Mesh[2].color = Color;

    Mesh[3].x = Size; Mesh[3].y = -Size;  Mesh[3].z = -Size;
    Mesh[3].u = 1.0f; Mesh[3].v = 1.0f;
    Mesh[3].color = Color;

    mpD3D8Device->SetTexture(0, mpSkybox[5]);
    mpD3D8Device->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, Mesh, sizeof(LitVertex));

    //left
    Mesh[0].x = -Size;  Mesh[0].y = Size;  Mesh[0].z = -Size;
    Mesh[0].u = 0.0f;   Mesh[0].v = 0.0f;
    Mesh[0].color = Color;

    Mesh[1].x = -Size;  Mesh[1].y = Size;  Mesh[1].z = Size;
    Mesh[1].u = 1.0f;   Mesh[1].v = 0.0f;
    Mesh[1].color = Color;

    Mesh[2].x = -Size;  Mesh[2].y = -Size; Mesh[2].z = -Size;
    Mesh[2].u = 0.0f;   Mesh[2].v = 1.0f;
    Mesh[2].color = Color;

    Mesh[3].x = -Size;  Mesh[3].y = -Size; Mesh[3].z = Size;
    Mesh[3].u = 1.0f;   Mesh[3].v = 1.0f;
    Mesh[3].color = Color;

    mpD3D8Device->SetTexture(0, mpSkybox[4]);
    mpD3D8Device->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, Mesh, sizeof(LitVertex));

    //restore view matrix
    mpD3D8Device->SetTransform(D3DTS_VIEW, view);

    // Restore the textures
    mpD3D8Device->SetTextureStageState(0, D3DTSS_COLOROP,   D3DTOP_MODULATE);

    mpD3D8Device->SetRenderState(D3DRS_LIGHTING, TRUE);
    mpD3D8Device->SetRenderState(D3DRS_ZWRITEENABLE, TRUE);
    mpD3D8Device->SetRenderState(D3DRS_CULLMODE, D3DCULL_CCW);
    mpD3D8Device->SetTextureStageState(0, D3DTSS_ADDRESSU, D3DTADDRESS_WRAP);
    mpD3D8Device->SetTextureStageState(0, D3DTSS_ADDRESSV, D3DTADDRESS_WRAP);

    return true;
}

